#include "TSHMSCerenkov.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Orb.hh"
#include "G4Ellipsoid.hh"
#include "G4Paraboloid.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVParameterised.hh"

#include "G4VSolid.hh"
#include "G4SubtractionSolid.hh"
#include "G4UnionSolid.hh"
#include "G4IntersectionSolid.hh"

#include "G4UserLimits.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4ios.hh"

#include "G4NistManager.hh"
#include "G4Sphere.hh"
#include "G4OpticalSurface.hh"
#include "G4LogicalBorderSurface.hh"
#include "TSHMSMaterials.hh"

#include "G4LogicalSkinSurface.hh"
#include "G4OpticalSurface.hh"
#include "G4LogicalBorderSurface.hh"
#include "G4MaterialPropertiesTable.hh"
#include "G4SDManager.hh"
#include "TSHMSPmtSD.hh"
#include "G4VSensitiveDetector.hh"
#include "G4SubtractionSolid.hh"

#include "TSHMSAnalysisManager.hh"
#include "G4ThreeVector.hh"

#include <string>
TSHMSPmtSD* TSHMSCerenkov::aPmtSD;

using namespace std;
//------------------------------------------------------------------------------
TSHMSCerenkov::TSHMSCerenkov()
  :solidCerTubs(0),  logicCerTubs(0),  physiCerTubs(0),
   solidCerMirror(0),logicCerMirror(0),   physiCerMirror(0),
   solidCerPMT(0),   logicCerPMT(0),   physiCerPMT(0),
   solidPMTCathode(0), logicPMTCathode(0), physiPMTCathode(0)
{
  analysis = TSHMSAnalysisManager::getInstance();
  TSHMSCerenkov::Init();
}
//------------------------------------------------------------------------------
TSHMSCerenkov::~TSHMSCerenkov()
{
  analysis->fout_cer.close();
  analysis->fout_hit.close();
}
//------------------------------------------------------------------------------ 
G4VPhysicalVolume* TSHMSCerenkov::ConstructCerenkov( G4VPhysicalVolume *physiWorld , G4LogicalVolume *logicWorld , G4ThreeVector CerPosThreeVector )
{
  //G4VSensitiveDetector* aPmtSD;


  TSHMSMaterialManager* matManager = TSHMSMaterialManager::getInstance();
  G4Material* cergas    = matManager->FindMaterial(G4String("C4F8O"));
  G4Material* AlMirror  = matManager->FindMaterial(G4String("AlMirror"));
  G4Material* UVGlass   = matManager->FindMaterial(G4String("UVGlass"));
  G4Material* AlTank    = matManager->FindMaterial(G4String("AlTank"));
  G4Material* AlCathode    = matManager->FindMaterial(G4String("AlCathode"));
  G4Material* Al = new G4Material("Al",13,26.98*g/mole,2.7*g/cm3);

  G4ThreeVector zero3v( 0.0 , 0.0 , 0.0 );

  if( !cergas ) { G4cerr << "Material C4F8O is not defined" << G4endl; exit( EXIT_FAILURE ); }


//------------------------------------------------------------------------------ 
// Begin Tank Construction
//------------------------------------------------------------------------------

  G4double CerRad = TankInnerDiameter / 2.0 *cm;
  G4double CerLen = TankLength / 2.0 * cm;
  G4double CerPro = 2*cm;

  // Here is the creation of the Cerenkov tank
  solidCerTubs   =  new G4Tubs("CerTube" , 0.0 , CerRad ,CerLen , 0.0 , twopi );
  G4SubtractionSolid *solidCerTubsOneWindow;
  G4SubtractionSolid *solidCerTubsTwoWindow;

  G4double CutOutRad = (CerPro*CerPro + CerRad*CerRad)/(2*CerPro);
  G4double Sweep = asin(CerRad/CutOutRad);
  G4Sphere* solidWindowOne = new G4Sphere("solidwindow", CutOutRad-CerPro , CutOutRad , 0, 360*degree, 0 , 1.0001*Sweep );
  G4Sphere* solidWindowTwo = new G4Sphere("solidwindow", CutOutRad-CerPro , CutOutRad , 0, 360*degree, 180*degree - 0.9999*Sweep , 180*degree );

  solidCerTubsOneWindow = new G4SubtractionSolid("solidCerTubsOneWindow", solidCerTubs, solidWindowOne , 0, G4ThreeVector(0.0 , 0.0, -CutOutRad-CerLen+CerPro ) );
  solidCerTubsTwoWindow = new G4SubtractionSolid("solidCerTubsTwoWindow", solidCerTubsOneWindow, solidWindowTwo, 0, G4ThreeVector(0.0, 0.0, +CutOutRad+CerLen-CerPro));

  logicCerTubs   =  new G4LogicalVolume( solidCerTubsTwoWindow , cergas , "CerLog", 0 , 0 , 0 );
  //logicCerTubs   =  new G4LogicalVolume( solidCerTubs , cergas , "CerLog", 0 , 0 , 0 );
  physiCerTubs   =  new G4PVPlacement( 0 , CerPosThreeVector , logicCerTubs , "cer" , logicWorld , false , 0 );

  //G4LogicalVolume* logicTest   =  new G4LogicalVolume( solidWindowTwo , cergas , "TestLog", 0 , 0 , 0 );
  //G4VPhysicalVolume* physiTest   =  new G4PVPlacement( 0 , G4ThreeVector(CerPosThreeVector.getX(),CerPosThreeVector.getY(),CerPosThreeVector.getZ()+CerLen+CutOutRad-CerPro) , logicTest , "test" , logicWorld , false , 0 );
  
  // Here we have the charateristics if the Cerenkov Tank
  // Tank Surface Properties
  G4double PhotonEnergyTank[2] = { 1.2*eV, 10*eV };
  G4double ReflectivityTank[2] = { 0.2, 0.2 };
  
  if(PMTon) {
  G4OpticalSurface *CerTank = new G4OpticalSurface("CerTank");
  CerTank->SetType(dielectric_metal);
  CerTank->SetFinish(polished);
  CerTank->SetModel(glisur);

  G4MaterialPropertiesTable *CerTankTb = new G4MaterialPropertiesTable();
  CerTankTb->AddProperty("REFLECTIVITY", PhotonEnergyTank , ReflectivityTank , 2 );
  CerTank->SetMaterialPropertiesTable(CerTankTb);
  
  CerTank->DumpInfo();

  G4LogicalBorderSurface *CerTankSurf;
  CerTankSurf = new G4LogicalBorderSurface("Inner Tank Surface" , physiCerTubs, physiWorld , CerTank );

  }
//------------------------------------------------------------------------------
// End Tank Construction
//------------------------------------------------------------------------------








//------------------------------------------------------------------------------ 
// Begin Mirror Construction 
//------------------------------------------------------------------------------

  G4double MirrRadii  = MirrorR * cm;
  G4double MirrThikn  = 0.3 * cm;

  //G4Box *solidBox[4];
  //G4LogicalVolume *logicBox[4];
  //G4VPhysicalVolume *physiBox[4];
  //-- Rotation matrix for PMT inside Cerenkov
  G4RotationMatrix *MirrorRotation[4];
  G4RotationMatrix *PMTRotation[4];
  G4IntersectionSolid *solidBoxSphInter[4];
  G4LogicalVolume *logicMirror[4];
  G4double sphSize[] = { MirrRadii , MirrThikn };
  char logicMirrorName[10];
  for(int mirror=0; mirror<4; mirror++) {
	// Create a new rotation matrix and set the X and Y rotation compomentsa
	// of the mirrors and the PMT tubes
	
  	MirrorRotation[mirror] = new G4RotationMatrix;
  	MirrorRotation[mirror]->rotateX(0.0 *degree);
  	MirrorRotation[mirror]->rotateY(MirrorAngle[mirror] *degree);
	if(!MirrorSphere) {
  		MirrorRotation[mirror]->rotateY(180.0 *degree);
  	}	
	PMTRotation[mirror] = new G4RotationMatrix;
  	PMTRotation[mirror]->rotateX(0.0 *degree);
  	PMTRotation[mirror]->rotateY(PMTAngle[mirror] *degree);
	
	// Create logical mirror form the intercetion solid
	sprintf(logicMirrorName,"CerMirr%d_log",mirror+1);
	if(MirrorSphere) {
    	solidBoxSphInter[mirror] = TSHMSCerenkov::MakeMirrorSolid( MirrorWidth[mirror] , sphSize );
	} else {
		solidBoxSphInter[mirror] = TSHMSCerenkov::MakeMirrorParabola( MirrorWidth[mirror] , sphSize );
	}
    logicMirror[mirror] = new G4LogicalVolume(solidBoxSphInter[mirror], AlMirror, logicMirrorName, 0, 0, 0 );

    //solidBox[mirror] = new G4Box("mirbox", MirrorWidth[mirror][0] , MirrorWidth[mirror][1] , MirrorWidth[mirror][2] );
	//logicBox[mirror] = new G4LogicalVolume(solidBox[mirror], cergas, "this", 0, 0, 0);
  }

  /* Here we will begin with the mirror and placement, the values posX, posY, posZ are the
     center of the mirror volumes as placed with respect to the cherenkov vessle. The center
     of the vessel is (0,0,0).
  */ 

  G4VPhysicalVolume *physiMirror[4];
  //G4double MirrorPosition[4][3];
  char volname[10];

  for(int mirror=0; mirror<4; mirror++) {
  		//physiBox[mirror] = new G4PVPlacement( MirrorRotation[mirror] , G4ThreeVector(analysis->TransportToTankX(MirrorPosition[mirror][X_AXIS]),analysis->TransportToTankY(MirrorPosition[mirror][Y_AXIS]),analysis->TransportToTankZ(MirrorPosition[mirror][Z_AXIS]) ), logicBox[mirror] , "that" , logicCerTubs , true , mirror );

		sprintf(volname,"mirr%d",mirror);
  		physiMirror[mirror] = new G4PVPlacement( MirrorRotation[mirror] , G4ThreeVector(analysis->TransportToTankX(MirrorPosition[mirror][X_AXIS]),analysis->TransportToTankY(MirrorPosition[mirror][Y_AXIS]),analysis->TransportToTankZ(MirrorPosition[mirror][Z_AXIS]) ), logicMirror[mirror] , volname , logicCerTubs , true , mirror );
  
		G4cout 	<< " Mirror 1 in Tank " 
	 			<< " x = " << MirrorPosition[mirror][X_AXIS] << "  " 
	 			<< " y = " << MirrorPosition[mirror][Y_AXIS] << "  " 
	 			<< " z = " << MirrorPosition[mirror][Z_AXIS]  << "  " 
	 			<< G4endl;
  		G4cout << " PMT 1 in Tank " 
	 			<< " x = " << PMTPosition[mirror][X_AXIS] << "  " 
	 			<< " y = " << PMTPosition[mirror][Y_AXIS] << "  " 
	 			<< " z = " << PMTPosition[mirror][Z_AXIS] << "  " 
	 			<< G4endl;
	  
  }

  // -- Optical surface between Cerenkov gas and mirror

  //  G4MaterialPropertiesTable* CerGasMPTable = cergas->GetMaterialPropertiesTable ();
  //  G4MaterialPropertyVector *CerGasMPTableVec = CerGasMPTable->GetProperty("RINDEX");
  //  G4cout << CerGasMPTableVec->GetMinPhotonMomentum()  << " " << CerGasMPTableVec->GetMaxPhotonMomentum() << G4endl;

  // Mirror Surface Properties
  G4double PhotonEnergyMirr[32] = 
    { 1.378*eV,2.067*eV,2.480*eV,2.755*eV,3.221*eV,3.435*eV,3.643*eV,3.802*eV,
      4.159*eV,4.510*eV,4.819*eV,4.960*eV,5.061*eV,5.166*eV,5.276*eV,5.391*eV,
      5.511*eV,5.636*eV,5.767*eV,5.904*eV,6.048*eV,6.200*eV,6.359*eV,6.526*eV,
      6.702*eV,6.889*eV,7.085*eV,7.294*eV,7.515*eV,7.750*eV,8.000*eV,8.266*eV };

  G4double ReflectivityMirr[32] = 
    { 0.920, 0.920, 0.920, 0.910, 0.900, 0.840, 0.830, 0.910,
      0.900, 0.900, 0.890, 0.869, 0.865, 0.869, 0.867, 0.871,
      0.870, 0.872, 0.870, 0.870, 0.865, 0.864, 0.863, 0.863,
      0.866, 0.864, 0.863, 0.857, 0.842, 0.831, 0.819, 0.808 };

  G4OpticalSurface *CerMirror = new G4OpticalSurface("CerMirror");
  //  CerMirror->SetModel( unified );
  CerMirror->SetType( dielectric_metal );
  CerMirror->SetFinish(polished);
  CerMirror->SetModel(glisur);

  G4MaterialPropertiesTable *CerMirrorTb = new G4MaterialPropertiesTable();
  CerMirrorTb->AddProperty("REFLECTIVITY", PhotonEnergyMirr , ReflectivityMirr , 32 );
  CerMirror->SetMaterialPropertiesTable(CerMirrorTb);
  
  CerMirror->DumpInfo();

  G4LogicalBorderSurface *CerMirrorSurf;
  CerMirrorSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiMirror[0] , CerMirror );
  CerMirrorSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiMirror[1] , CerMirror );
  CerMirrorSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiMirror[2] , CerMirror );
  CerMirrorSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiMirror[3] , CerMirror );

  // -- End Optical surface between Cerenkov gas and mirror

  logicMirror[0]->SetVisAttributes(new G4VisAttributes(G4Colour::Blue()));
  logicMirror[1]->SetVisAttributes(new G4VisAttributes(G4Colour::Red()));
  logicMirror[2]->SetVisAttributes(new G4VisAttributes(G4Colour::Yellow()));
  logicMirror[3]->SetVisAttributes(new G4VisAttributes(G4Colour::Magenta()));

//------------------------------------------------------------------------------ 
// End Mirror Construction 
//------------------------------------------------------------------------------










//------------------------------------------------------------------------------ 
// Begin PMT Construction 
//------------------------------------------------------------------------------
  
  G4double ToWindow = 0.635*cm;
  G4double Cathode_L = 0.100*cm;

  G4double PMTWindowPos[4][3];
  G4double PMTCathodePos[4][3];
  G4double PMTFlangePos[4][3];
  G4double PMTBackingPos[4][3];

  for(int mirror=0; mirror<4; mirror++) {

  	analysis->PMT[mirror][X_AXIS] = PMTPosition[mirror][X_AXIS];
  	analysis->PMT[mirror][Y_AXIS] = PMTPosition[mirror][Y_AXIS];
  	analysis->PMT[mirror][Z_AXIS] = PMTPosition[mirror][Z_AXIS]-CerLen;
  	PMTPosition[mirror][Z_AXIS] -= CerLen;

  	analysis->PMTAng[mirror] = PMTAngle[mirror];
 
	PMTCathodePos[mirror][0] = PMTPosition[mirror][0] + 1.5*Cathode_L*(sin(PMTAngle[mirror] *degree));
  	PMTCathodePos[mirror][1] = PMTPosition[mirror][1];
  	PMTCathodePos[mirror][2] = PMTPosition[mirror][2] - 1.5*Cathode_L*(cos(PMTAngle[mirror] *degree));

  	PMTFlangePos[mirror][0] = PMTPosition[mirror][0] - ToWindow*(sin(PMTAngle[mirror] *degree));
  	PMTFlangePos[mirror][1] = PMTPosition[mirror][1];
  	PMTFlangePos[mirror][2] = PMTPosition[mirror][2] + ToWindow*(cos(PMTAngle[mirror] *degree));
  			
	PMTWindowPos[mirror][0] = PMTPosition[mirror][0] - PMT_L*(sin(PMTAngle[mirror] *degree));
  	PMTWindowPos[mirror][1] = PMTPosition[mirror][1];
  	PMTWindowPos[mirror][2] = PMTPosition[mirror][2] + PMT_L*(cos(PMTAngle[mirror] *degree));
	
	PMTBackingPos[mirror][0] = PMTPosition[mirror][0] + (2*ToWindow+Cathode_L)*(sin(PMTAngle[mirror] *degree));
  	PMTBackingPos[mirror][1] = PMTPosition[mirror][1];
  	PMTBackingPos[mirror][2] = PMTPosition[mirror][2] - (2*ToWindow+Cathode_L)*(cos(PMTAngle[mirror] *degree));
  }

  analysis->PMTR = PMT_R*cm;
  Window_R = 8.0;

  G4Tubs* solidPMTWindow  =  new G4Tubs("solidPMTWindow", 0.0 , Window_R *cm , PMT_L , 0.0 , twopi );
  G4Tubs* solidPMTFlange  =  new G4Tubs("solidPMTFlange", Window_R *cm , 13.27 *cm , 1.9*cm , 0.0 , twopi );
  G4Tubs* solidPMTCathode  =  new G4Tubs("solidPMTCathode", 0.0 , PMT_R *cm , Cathode_L , 0.0 , twopi );
  G4Tubs* solidPMTBacking = new G4Tubs("solidPMTBacking", 0.0 , 13.27 *cm , 0.1*cm , 0.0 , twopi );

  G4double PhotonEnergyC4F8O[] = { 2.0*eV, 3.0*eV, 4.0*eV, 5.0*eV, 6.0*eV }; // fix me, how to get this from TSHMSMaterials.cc
 
	G4LogicalVolume* logicPMTCathode = new G4LogicalVolume( solidPMTCathode, Al, "CathodeLog", 0 , 0 , 0 );
  	G4LogicalVolume* logicPMTFlange   =  new G4LogicalVolume( solidPMTFlange , AlTank, "FlangeLog", 0 , 0 , 0 ); 
  	G4LogicalVolume* logicPMTBacking   =  new G4LogicalVolume( solidPMTBacking , cergas, "BackingLog", 0 , 0 , 0 ); 
  	G4LogicalVolume* logicPMTWindow   =  new G4LogicalVolume( solidPMTWindow , UVGlass , "PmtLog", 0 , 0 , 0 ); // logicPMTWindow is used for the
																											 // PMT quartz window, this needs to have
    logicPMTCathode->SetVisAttributes(new G4VisAttributes(G4Colour::Red()));
    logicPMTFlange->SetVisAttributes(new G4VisAttributes(G4Colour::Cyan()));
    logicPMTWindow->SetVisAttributes(new G4VisAttributes(G4Colour::Blue()));
																											 // it's own volumn and radius in the future
  	//G4LogicalVolume* logicPMTWindow   =  new G4LogicalVolume( solidPMTWindow , cergas , "PmtLog", 0 , 0 , 0 );

	// Since we have the PMT positioning in transport coordinates, we want to place then in the tank in tank coordinates
	
  //------------------------------------------------ 
  // Sensitive detectors
  //------------------------------------------------ 


   G4SDManager* SDman = G4SDManager::GetSDMpointer();
   aPmtSD = new TSHMSPmtSD(G4String("PmtSD"));
   SDman->AddNewDetector(aPmtSD);
   logicPMTCathode->SetSensitiveDetector(aPmtSD);

   if(PMTon) {
  //


	G4VPhysicalVolume *physiPMTWindow[4];
	G4VPhysicalVolume *physiPMTFlange[4];
	G4VPhysicalVolume *physiPMTCathode[4];
	G4VPhysicalVolume *physiPMTBacking[4];

	for(int mirror=0; mirror<4; mirror++) {
		sprintf(volname,"window%d",mirror+1);
  		physiPMTWindow[mirror] = new G4PVPlacement(PMTRotation[mirror],G4ThreeVector(PMTWindowPos[mirror][0]-5.0*cm,PMTWindowPos[mirror][1],PMTWindowPos[mirror][2]),logicPMTWindow,volname,logicCerTubs,false,mirror);
		
		sprintf(volname,"flange%d",mirror+1);
  		physiPMTFlange[mirror] = new G4PVPlacement(PMTRotation[mirror],G4ThreeVector(PMTFlangePos[mirror][0]-5.0*cm,PMTFlangePos[mirror][1],PMTFlangePos[mirror][2]),logicPMTFlange,volname,logicCerTubs,false,mirror);
		
		sprintf(volname,"cathode%d",mirror+1);
  		physiPMTCathode[mirror] = new G4PVPlacement(PMTRotation[mirror],G4ThreeVector(PMTCathodePos[mirror][0]-5.0*cm,PMTCathodePos[mirror][1],PMTCathodePos[mirror][2]),logicPMTCathode,volname,logicCerTubs,false,mirror);
		
		sprintf(volname,"backing%d",mirror+1);
  		physiPMTBacking[mirror] = new G4PVPlacement(PMTRotation[mirror],G4ThreeVector(PMTBackingPos[mirror][0]-5.0*cm,PMTBackingPos[mirror][1],PMTBackingPos[mirror][2]),logicPMTBacking,volname,logicCerTubs,false,mirror);
	}

  G4double PhotonEnergyPMTCathode[10] = 
    { 1.771*eV, 1.968*eV, 2.066*eV, 2.254*eV, 2.480*eV,
	  2.695*eV, 3.100*eV, 6.199*eV, 7.085*eV, 7.749*eV };

  G4double ReflectivityPMTCathode[10] = 
	{ 0.000, 0.002, 0.010, 0.068, 0.150,
	  0.200, 0.246, 0.217, 0.140, 0.006 };

  G4OpticalSurface *CerCathode = new G4OpticalSurface("CerCathode");
  //  CerCathode->SetModel( unified );
  CerCathode->SetType( dielectric_metal );
  CerCathode->SetFinish(polished);
  CerCathode->SetModel(glisur);

  G4MaterialPropertiesTable *CerCathodeTb = new G4MaterialPropertiesTable();
  CerCathodeTb->AddProperty("REFLECTIVITY", PhotonEnergyPMTCathode , ReflectivityPMTCathode , 10 );
  CerCathode->SetMaterialPropertiesTable(CerCathodeTb);
  
  CerCathode->DumpInfo();

  G4LogicalBorderSurface *CerCathodeSurf;
  CerCathodeSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiPMTCathode[0] , CerCathode );
  CerCathodeSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiPMTCathode[1] , CerCathode );
  CerCathodeSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiPMTCathode[2] , CerCathode );
  CerCathodeSurf = new G4LogicalBorderSurface("Cer gas/Cer Mirror Surface" , physiCerTubs , physiPMTCathode[3] , CerCathode );

	// Flange Surface Properties
 	//G4double PhotonEnergyTank[2] = { 1.2*eV, 10*eV };
	//G4double ReflectivityTank[2] = { 0.2, 0.2 };
  
 	G4OpticalSurface *CerFlange = new G4OpticalSurface("CerFlange");
 	CerFlange->SetType(dielectric_metal);
 	CerFlange->SetFinish(polished);
 	CerFlange->SetModel(glisur);

 	G4MaterialPropertiesTable *CerFlangeTb = new G4MaterialPropertiesTable();
 	CerFlangeTb->AddProperty("REFLECTIVITY", PhotonEnergyTank , ReflectivityTank , 2 );
 	CerFlange->SetMaterialPropertiesTable(CerFlangeTb);
  
	CerFlange->DumpInfo();

 	G4LogicalBorderSurface *CerFlangeSurf;
 	CerFlangeSurf = new G4LogicalBorderSurface("Inner Tank Surface" , physiCerTubs, physiPMTFlange[0] , CerFlange );
 	CerFlangeSurf = new G4LogicalBorderSurface("Inner Tank Surface" , physiCerTubs, physiPMTFlange[1] , CerFlange );
 	CerFlangeSurf = new G4LogicalBorderSurface("Inner Tank Surface" , physiCerTubs, physiPMTFlange[1] , CerFlange );
 	CerFlangeSurf = new G4LogicalBorderSurface("Inner Tank Surface" , physiCerTubs, physiPMTFlange[1] , CerFlange );



	// -- Set properties for the PMTWindow
    //G4double ReflectivityUVglass[5] = { 0.2 , 0.2 , 0.2 , 0.2 , 0.2 };
  	//G4double RefractIndexUVglass[5] = { 1.468 , 1.468 , 1.468 , 1.468 , 1.468 };
  	//G4double SpecularlobeUVGlass [5]  ={ 0.3, 0.3, 0.3, 0.3, 0.3 }; // reflection probabilty about the normal of a micro facet.
  	//G4double SpecularspikeUVGlass[5] = { 0.2, 0.2, 0.2, 0.2, 0.2 }; // probability of reflection about the average surface normal.
  	//G4double BackscatterUVGlass  [5] = { 0.1, 0.1, 0.1, 0.1, 0.1 }; // several reflections within a deep groove with the ultimate result of exact back-scattering.
  	//G4double EfficiencyUVGlass   [5] = { 0.8, 0.8, 0.8, 0.8, 0.8 }; // 


  	// Create the PMT window surface here 
  	G4OpticalSurface *CerToWindow = new G4OpticalSurface("CerToWindow");
  
  	CerToWindow->SetType( dielectric_dielectric );

  	//  CerToWindow->SetModel( unified );
  	CerToWindow->SetModel( glisur );
  	CerToWindow->SetFinish( polished );
  	//  CerToWindow->SetSigmaAlpha(sigma_alpha);
  
  	const G4int NUM = 14;
  	G4double pp[NUM] = { 1.653*eV, 4.133*eV, 4.275*eV, 4.428*eV, 4.592*eV, 4.769*eV, 4.959*eV, 5.061*eV, 5.209*eV, 5.391*eV, 5.767*eV, 6.525*eV, 6.888*eV, 12.398*eV };
  	G4double eff[NUM] = { 0.0,  0.0,  0.1,  0.55,  0.68,  0.66,  0.64,  0.68,  0.78,  0.84,  0.88,  0.91,  0.92,  0.92 };
  	G4double ref[NUM] = { 1.0,  1.0,  0.9,  0.45,  0.32,  0.34,  0.36,  0.32,  0.22,  0.16,  0.12,  0.09,  0.08,  0.08 };
  	G4double rindex[NUM] = { 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585 };

	G4MaterialPropertiesTable *CerToWindowTb = new G4MaterialPropertiesTable();
  	//CerToWindowTb->AddProperty("REFLECTIVITY"         , PhotonEnergyC4F8O , ReflectivityUVglass  , 5 );
  	//CerToWindowTb->AddProperty("RINDEX"               , PhotonEnergyC4F8O , RefractIndexUVglass  , 5 );
  	//CerToWindowTb->AddProperty("SPECULARLOBECONSTANT" , PhotonEnergyC4F8O , SpecularlobeUVGlass  , 5 );
  	//CerToWindowTb->AddProperty("SPECULARSPIKECONSTANT", PhotonEnergyC4F8O , SpecularspikeUVGlass , 5 );
  	//CerToWindowTb->AddProperty("BACKSCATTERCONSTANT"  , PhotonEnergyC4F8O , BackscatterUVGlass   , 5 );
  	//CerToWindowTb->AddProperty("EFFICIENCY"           , PhotonEnergyC4F8O , EfficiencyUVGlass    , 5 );
  	CerToWindowTb->AddProperty("REFLECTIVITY"         , pp , ref , NUM );
  	CerToWindowTb->AddProperty("RINDEX"               , pp , rindex  , NUM );
  	CerToWindowTb->AddProperty("EFFICIENCY"           , pp , eff  , NUM );
  
 
	G4LogicalBorderSurface *CerToWindowSurf;
  	CerToWindowSurf = new G4LogicalBorderSurface("Cer gas/Cer PMT1 Surface" , physiCerTubs , physiPMTWindow[0] , CerToWindow );
  	CerToWindowSurf = new G4LogicalBorderSurface("Cer gas/Cer PMT2 Surface" , physiCerTubs , physiPMTWindow[1] , CerToWindow );
  	CerToWindowSurf = new G4LogicalBorderSurface("Cer gas/Cer PMT3 Surface" , physiCerTubs , physiPMTWindow[2] , CerToWindow );
  	CerToWindowSurf = new G4LogicalBorderSurface("Cer gas/Cer PMT4 Surface" , physiCerTubs , physiPMTWindow[3] , CerToWindow );
	CerToWindow->SetMaterialPropertiesTable(CerToWindowTb);
 
  	G4cout << " Start UVGlass " << G4endl;
  	CerToWindow->DumpInfo();
  	G4cout << " End   UVGlass " << G4endl;

  // -- End Optical surface between Cerenkov gas and window



 }

//------------------------------------------------------------------------------ 
// End PMT Construction 
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------ 
// Start Plane Construction 
//------------------------------------------------------------------------------
  G4double PMT_Tank[4][3];

  // ----- Planes at the front of cerenkov
  for(int mirror=0; mirror<4; mirror++) {
		PMT_Tank[mirror][X_AXIS] = analysis->TransportToTankX(PMTPosition[mirror][X_AXIS]);
		PMT_Tank[mirror][Y_AXIS] = analysis->TransportToTankY(PMTPosition[mirror][Y_AXIS]);
		PMT_Tank[mirror][Z_AXIS] = analysis->TransportToTankZ(PMTPosition[mirror][Z_AXIS]);
  
  }

  TSHMSCerenkov::MakePlanes( PMT_Tank , PMTRotation[0] , NumPlanes , PlaneDistance *cm );
  TSHMSCerenkov::MakePlanes( PMT_Tank , PMTRotation[1] , NumPlanes , PlaneDistance *cm );
  TSHMSCerenkov::MakePlanes( PMT_Tank , PMTRotation[2] , NumPlanes , PlaneDistance *cm );
  TSHMSCerenkov::MakePlanes( PMT_Tank , PMTRotation[3] , NumPlanes , PlaneDistance *cm );
  
  // Backplane used to catch all the missed photons
  TSHMSCerenkov::BackPlane( CerLen - CerPro , CerRad);
  // ----- End Planes
//------------------------------------------------------------------------------ 
// End Plane Construction 
//------------------------------------------------------------------------------

  logicCerTubs->SetVisAttributes(new G4VisAttributes(G4Colour::Grey()));

  return physiCerTubs;
}
//-----------------------------------------------------------------------------------------------------
G4IntersectionSolid* TSHMSCerenkov::MakeMirrorSolid( G4double *boxSize , G4double *sphSize )
{
  G4double dx_width = boxSize[0];
  G4double dy_width = boxSize[1];
  G4double dz_width = boxSize[2];

  G4Box    *solidBox  = new G4Box("mirbox", dx_width , dy_width , dz_width );
  
  G4double MirrRadii = sphSize[0];
  G4double MirrThikn = sphSize[1];

  G4double L = sqrt( dx_width * dx_width + dy_width * dy_width );

  G4double MirrTheta = asin( L / MirrRadii );

  G4Sphere *solidSphere = new G4Sphere("mirSph", MirrRadii , MirrRadii + MirrThikn , 0.0 , twopi , 0 , MirrTheta );

  G4IntersectionSolid* solidBoxSphInter;

  solidBoxSphInter = new G4IntersectionSolid("boxsphint" , solidBox , solidSphere , 0 , G4ThreeVector( 0.0 , 0.0 , -MirrRadii - MirrThikn/2.0 ) );

  return solidBoxSphInter;
}
//-----------------------------------------------------------------------------------------------------
G4IntersectionSolid* TSHMSCerenkov::MakeMirrorParabola( G4double *boxSize , G4double *sphSize )
{
  G4double dx_width = boxSize[0];
  G4double dy_width = boxSize[1];
  G4double dz_width = boxSize[2];
  
  G4double MirrRadii = sphSize[0];
  G4double MirrThikn = sphSize[1];
  
  G4Box    *solidBox  = new G4Box("mirbox", dx_width , dy_width , dz_width );
  
  G4double MirrFocus = MirrRadii/2;
  
  G4double R1 = 0.0;
  G4double R2 = 1.001*sqrt(dx_width*dx_width + dy_width*dy_width);
  G4double Dz = 1.001*(1/(4*MirrRadii))*R2*R2+MirrThikn;

  
  G4Paraboloid *InnerParabola = new G4Paraboloid("innerPar", Dz, R1, R2);
  G4Paraboloid *OuterParabola = new G4Paraboloid("outerPar", Dz, R1, R2);
  G4SubtractionSolid* solidParabolaSub;
  G4IntersectionSolid* solidParabolaInter;

  solidParabolaSub = new G4SubtractionSolid("parsub", OuterParabola, InnerParabola, 0, G4ThreeVector( 0.0 , 0.0 , MirrThikn));
  solidParabolaInter = new G4IntersectionSolid("boxsphint" , solidBox , solidParabolaSub , 0, G4ThreeVector( 0.0 , 0.0 , dz_width/2.0-MirrThikn/2.0));

  return solidParabolaInter;
}
//-----------------------------------------------------------------------------------------------------
G4Box* TSHMSCerenkov::MakeMirrorSolidTest( G4double *boxSize , G4double * )
{
  G4double dx_width = boxSize[0];
  G4double dy_width = boxSize[1];
  G4double dz_width = boxSize[2];

  G4Box    *solidBox  = new G4Box("mirbox", dx_width , dy_width , dz_width );
  
  return solidBox;
}
//-----------------------------------------------------------------------------------------------------
G4Paraboloid* TSHMSCerenkov::MakeMirrorParabolaTest( G4double *boxSize , G4double *sphSize )
{
  G4double dx_width = boxSize[0];
  G4double dy_width = boxSize[1];
  G4double dz_width = boxSize[2];

  //G4Box    *solidBox  = new G4Box("mirbox", dx_width , dy_width , dz_width );
  
  G4double MirrRadii = sphSize[0];
  //G4double MirrThikn = sphSize[1];

  G4double MirrFocus = MirrRadii/2.0;

  G4double Dz = dz_width;
  G4double R1 = 0.0;
  G4double R2 = sqrt(4*MirrFocus*Dz);

  G4Paraboloid *solidParabola = new G4Paraboloid("mirPar", Dz, R1, R2);

  //G4IntersectionSolid* solidBoxSphInter;

  //solidBoxSphInter = new G4IntersectionSolid("boxsphint" , solidParabola , solidBox , 0 , G4ThreeVector( 0.0 , 0.0 , -MirrRadii - MirrThikn/2.0 ) );

  return solidParabola;
}
//-----------------------------------------------------------------------------------------------------
void TSHMSCerenkov::MakePlanes( G4double PMT_pos[][3] , G4RotationMatrix *PMTrot , G4int NPlanes , G4double DistOfPl )
{
  static G4int ncalls = 1;
  TSHMSMaterialManager* matManager = TSHMSMaterialManager::getInstance();
  G4Material* cergas    = matManager->FindMaterial(G4String("C4F8O"));
  G4double CerRad = TankInnerDiameter / 2.0 *cm;

  G4VisAttributes* pColors[4];

  pColors[0]=new G4VisAttributes(G4Colour::Blue());
  pColors[1]=new G4VisAttributes(G4Colour::Red());
  pColors[2]=new G4VisAttributes(G4Colour::Yellow());
  pColors[3]=new G4VisAttributes(G4Colour::Magenta());

  G4double MirrTiltAngle = PMTrot->thetaZ() / 2.0;

  G4double TubeAngle[] = { 0.0 , 1.5*pi , 0.5*pi , pi };
  char name[20];

  sprintf( name , "CerPlane%d" , ncalls );

  G4Tubs* CerPlane = new G4Tubs( name , 0.0 , CerRad / cos( fabs( 2.0 * MirrTiltAngle ) ) , 0.001 * cm , TubeAngle[ncalls-1] , pi / 2.0 );

  sprintf( name , "CerPlane%dLog" , ncalls );  
  G4LogicalVolume* logicCerPlane =  new G4LogicalVolume( CerPlane , cergas , name , 0 , 0 , 0 );

  logicCerPlane->SetVisAttributes(pColors[ncalls-1]);

  G4VPhysicalVolume *physCerPlane;

  G4int middle = (G4int)(NPlanes/2);
  G4int start = 50-middle;

  // --
  G4double PlaneXpos = fabs(PMT_pos[ncalls-1][0]);
  G4double PlaneZpos = PMT_pos[ncalls-1][2];
  G4double PlaneZ0   = PlaneZpos - fabs ( PlaneXpos ) * tan( 2.0 * fabs( MirrTiltAngle ) ) - middle*DistOfPl ;
  for( int ii = start; ii <= NPlanes+start-1 ; ii++ ) {
    sprintf( name , "CerPl%d_%d" , ncalls , ii );  
    physCerPlane = new G4PVPlacement( PMTrot , G4ThreeVector( 0.0 ,0.0 , PlaneZ0 + DistOfPl*(ii-start) ), logicCerPlane , name , logicCerTubs , false , 0 );
  }
  ncalls++;
}
void TSHMSCerenkov::BackPlane(G4double zpos, G4double tubeRad)
{
  TSHMSMaterialManager* matManager = TSHMSMaterialManager::getInstance();
  G4Material* cergas = matManager->FindMaterial(G4String("C4F8O"));

  char name[20];

  sprintf(name,"BackPlane");
  G4Tubs* CerPlane = new G4Tubs(name,0.0,tubeRad,0.001*cm,0.0,pi*2.0);

  sprintf(name,"BackPlaneLog");  
  G4LogicalVolume* logicCerPlane =  new G4LogicalVolume(CerPlane,cergas,name,0,0,0);

  G4VPhysicalVolume *physCerPlane;

  printf(name,"BackPl");  
  physCerPlane = new G4PVPlacement(0,G4ThreeVector(0.0 ,0.0, zpos),logicCerPlane,name,logicCerTubs,false,0);
}
void TSHMSCerenkov::Init()
{
  G4double  PMT1_ang;
  G4double  PMT2_ang;
  G4double  PMT3_ang;
  G4double  PMT4_ang;
  // -- Read input parameters from cer_config.dat

  G4cout << "-----------------------------------------------------" << G4endl;
  G4cout << "-------------------  Cerenkov properties  -----------" << G4endl;
  G4cout << "-----------------------------------------------------" << G4endl;
  string::size_type loc_beg;
  string line;
  ifstream fin("cer_config.dat" , ios::in );  
  if( !fin.is_open() ) { G4cerr << "Can't open file cer_config.dat " << G4endl; exit(1);}

  string::size_type Mtype;
  char particle[5];
  char sign; 
  
  getline( fin , line );
  getline( fin , line );
  getline( fin , line );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Mtype = line.find("b" , loc_beg);
 
  G4bool Binary; 
  
  if(Mtype != string::npos) {
      Binary = true;
  } else {
      Binary = false;
  }

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  FrontOfTank = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  analysis->SetTankFrontZ( FrontOfTank );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  TankLength = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  TankInnerDiameter = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  getline( fin , line );

  // Here we will determine if we are using spherical or parabolic mirrors

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Mtype = line.find("s",loc_beg);
  
  if(Mtype != string::npos) {
      MirrorSphere = true;
  } else {
      MirrorSphere = false;
  }
  string MirrorType = (MirrorSphere)? "Spherical" : "Parabolic";
  
  // Here we will determine if we are measuring from the corners or the middle 
  getline( fin , line );
  loc_beg = line.find(":",0);
  Mtype = line.find("c",loc_beg);
  
  if(Mtype != string::npos) {
      MirrorCorner = true;
  } else {
      MirrorCorner = false;
  }
  string MeasureType = (MirrorCorner)? "Measured from Corner to corner" : "Measured from middle to middle";
  
  // Get the mirror focal length
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  MirrorF = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  MirrorR = MirrorF * 2.0; 

  // -- Mirror tilt angles

  string subline;
 
  // New way of constructing the mirrors in the simulator 
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorStart[0][0], &MirrorStart[0][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorEnd[0][0], &MirrorEnd[0][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Mirror[0][1] = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorStart[1][0] , &MirrorStart[1][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorEnd[1][0] , &MirrorEnd[1][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Mirror[1][1] = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorStart[2][0] , &MirrorStart[2][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorEnd[2][0] , &MirrorEnd[2][2] );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Mirror[2][1] = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorStart[3][0] , &MirrorStart[3][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf" , &MirrorEnd[3][0] , &MirrorEnd[3][2] );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Mirror[3][1] = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  Overlap = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  
  getline( fin , line );
  getline( fin , line );
  
  // Here we turn the PMTs on or off 
  getline(fin , line ) ;
  loc_beg = line.find(":" , 0);
  Mtype = line.find("y" , loc_beg);
  
  if(Mtype != string::npos) {
      PMTon = true;
  } else {
      PMTon = false;
  }

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  PMT_R   = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  PMT_R = 0.9*PMT_R;


  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf, %lf" , &PMTPosition[0][0], &PMTPosition[0][1], &PMTPosition[0][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf, %lf" , &PMTPosition[1][0], &PMTPosition[1][1], &PMTPosition[1][2] );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf, %lf" , &PMTPosition[2][0], &PMTPosition[2][1], &PMTPosition[2][2] );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  subline = line.substr( loc_beg + 1 , ( line.size() - loc_beg ) );
  sscanf( subline.c_str() , "%lf, %lf, %lf" , &PMTPosition[3][0], &PMTPosition[3][1], &PMTPosition[3][2] );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  PMT1_ang = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  PMT2_ang = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  loc_beg = line.find(":" , 0);
  PMT3_ang = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
 
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  PMT4_ang = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  getline( fin , line );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  NumPlanes = (G4int)atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  
  getline( fin , line );
  loc_beg = line.find(":" , 0);
  PlaneDistance = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );

  getline( fin , line );
  getline( fin , line );
  getline( fin , line ); // -- Skip reading the cerenkov gas name and gas prasure parameters
  
  getline( fin , line ); // -- it is done in TSHMSMaterials.cc. 
  loc_beg = line.find(":", 0);
  G4double gasp = atof((line.substr(loc_beg + 1, (line.size()-loc_beg))).c_str());

  getline( fin , line );
  loc_beg  = line.find(":" , 0);
  G4double SHMS_P0  = atof( (line.substr( loc_beg + 1 , ( line.size() - loc_beg ) )).c_str() );
  analysis->SetSHMSP0( SHMS_P0 ); // will be used in PrimaryGeneratorAction

  
  getline(fin , line ) ;
  loc_beg = line.find(":" , 0);
  Mtype = line.find("k" , loc_beg);
  
  if(Mtype != string::npos) {
      sprintf(particle,"kaon"); 
  } else {
      sprintf(particle,"pi"); 
  }
  
  getline(fin , line ) ;
  loc_beg = line.find(":" , 0);
  Mtype = line.find("-" , loc_beg);
  
  if(Mtype != string::npos) {
	  sign = '-';
  } else {
	  sign = '+';
  }
  
  sprintf(analysis->Particle,"%s%c",particle,sign);

  fin.close();

  G4double MirrRadii  = MirrorR * cm;
  G4double MirrThikn  = 0.3 * cm;

  G4double SphereToParabola[4];

  G4double MirrorLength;
  G4double MirrorTheta;
  // ----------------- Here is were we perform a lot of the variable initialization
  // Make Mirror End and Start the half length and in units of cm 
  for(int mirror=0; mirror<4; mirror++) {
	  for(int axis=0; axis<3; axis++) {
		// We want to convert the entered dimensions into the proper
		// units (cm) -PS
  		MirrorEnd[mirror][axis] = MirrorEnd[mirror][axis] *cm;
  		MirrorStart[mirror][axis] = MirrorStart[mirror][axis] *cm;
		PMTPosition[mirror][axis] = PMTPosition[mirror][axis] *cm;

		// Here we are want to calculate the size of the mirror in the
		// x,y,z directions. Geant4 bases the size of solids on half
		// lengths hence the need to divide by two -PS
		if(axis == Y_AXIS) {
			// The size of the mirror along the y axis was given 
			// directly from the input file -PS
  			Mirror[mirror][axis] = fabs(Mirror[mirror][axis])/2.0 *cm;
		} else {
			// The size of the mirror along the x and z axis need
			// to be calculated based on their start and end points -PS
			Mirror[mirror][axis] = fabs(MirrorEnd[mirror][axis]-MirrorStart[mirror][axis])/2.0;
		}
	 } // end axis loop

	// We are getting the mirror tilt angles here
	// since is about the y axies then we
	// only need the dimentions of the x and z axis -PS
  	MirrorAngle[mirror] = atan2(Mirror[mirror][Z_AXIS],Mirror[mirror][X_AXIS])/degree;
	if((mirror == MIRROR_3)||(mirror == MIRROR_4)) {
		// mirrors 3 and 4 are at a negative angle
  		MirrorAngle[mirror] = -MirrorAngle[mirror];
	}
  
    // Here we are getting the size of the mirror here
	// we determine the length and width -PS
	MirrorWidth[mirror][X_AXIS] = Mirror[mirror][X_AXIS]/cos( MirrorAngle[mirror] *degree );
  	MirrorWidth[mirror][Y_AXIS] = Mirror[mirror][Y_AXIS];

	// Here we are finding the maximum length for the 
	// mirror tip to tip to find the mirror span Theta -PS
  	MirrorLength = sqrt(MirrorWidth[mirror][X_AXIS]*MirrorWidth[mirror][X_AXIS] + MirrorWidth[mirror][Y_AXIS]*MirrorWidth[mirror][Y_AXIS]);
    MirrorTheta = asin(MirrorLength/MirrRadii);

		// We can now determine the depth of the mirror in the z-direction
	if(MirrorSphere) {
  		MirrorWidth[mirror][Z_AXIS] = MirrRadii*(1.0-cos(MirrorTheta))+MirrThikn/2.0;
		SphereToParabola[mirror] = 0.0;
	} else {
		G4double SphereWidth = MirrRadii*(1.0-cos(MirrorTheta))+MirrThikn/2.0;
  		MirrorWidth[mirror][Z_AXIS] = ((1/(2*MirrRadii))*MirrorLength*MirrorLength)+MirrThikn;
		SphereToParabola[mirror] = SphereWidth - MirrorWidth[mirror][Z_AXIS];
	}	
	} // end mirror loop -PS

  // Here we are going to follow the geometry of the
  // mirrors to comply with the ray trace simulation
  // with the inputed dimensions being those trough
  // the middle of the mirrors -PS
  
  // Here we will place the mirror in accordance to 
  // the ray trace program, it requires a little math
  // the can be found in my labbook Paul Selles
  G4double Small_dz;
  G4double dist, distx, distz;

  if(!MirrorCorner) {
  	for(int mirror=0; mirror<4; mirror++) {
      MirrorTheta = asin(MirrorWidth[mirror][X_AXIS]/(MirrRadii)); 
	  
	  if(MirrorSphere) {
	  	Small_dz = MirrRadii*(1.0-cos(MirrorTheta)) + MirrThikn/2.0;
	  } else {
	  	Small_dz = ((1/(2*MirrRadii))*MirrorWidth[mirror][X_AXIS]*MirrorWidth[mirror][X_AXIS]) + MirrThikn;
	  }

      dist = MirrorWidth[mirror][Z_AXIS] - Small_dz - SphereToParabola[mirror];
	  distx = dist*(sin(MirrorAngle[mirror] *degree));
	  distz = dist*(cos(MirrorAngle[mirror] *degree));

	  MirrorStart[mirror][X_AXIS] = MirrorStart[mirror][X_AXIS] + distx;
	  MirrorEnd[mirror][X_AXIS] = MirrorEnd[mirror][X_AXIS] + distx;
      Mirror[mirror][X_AXIS] = fabs(MirrorEnd[mirror][X_AXIS]-MirrorStart[mirror][X_AXIS])/2.0;
	
	  MirrorStart[mirror][Z_AXIS] = MirrorStart[mirror][Z_AXIS] - distz;
	  MirrorEnd[mirror][Z_AXIS] = MirrorEnd[mirror][Z_AXIS] - distz;
      Mirror[mirror][Z_AXIS] = fabs(MirrorEnd[mirror][Z_AXIS]-MirrorStart[mirror][Z_AXIS])/2.0;
	} // end mirror loop
  } // endif Not MirrorCorners

  PMTAngle[0] = PMT1_ang; 
  PMTAngle[1] = PMT2_ang; 
  PMTAngle[2] = -PMT3_ang; 
  PMTAngle[3] = -PMT4_ang; 
  

  // ----- Done with much of the initialization

  
  // Output the information for the user to see 
  G4cout << "Front of Cerenkov Tank:   " << FrontOfTank << G4endl
	 << "Cerenkov Tank length:   "   << TankLength << G4endl
	 << "Minimum distance from mirror to back window:  " << DisMirToBackWindow << G4endl
	 << "Tank inner diameter:  " << TankInnerDiameter << G4endl
	 << "Mirror radius of curvature:  " << MirrorR << G4endl
     << MirrorType << " " << MeasureType << G4endl
     << "Mirror1 corner1 (x,z) " << MirrorStart[0][0]/cm << "," << MirrorStart[0][2]/cm << G4endl
     << "Mirror1 corner2 (x,z) " << MirrorEnd[0][0]/cm << "," << MirrorEnd[0][2]/cm << G4endl
     << "Mirror2 corner1 (x,z) " << MirrorStart[1][0]/cm  << "," << MirrorStart[1][2]/cm  << G4endl
     << "Mirror2 corner2 (x,z) " << MirrorEnd[1][0]/cm  << "," << MirrorEnd[1][2]/cm  << G4endl
     << "Mirror3 corner1 (x,z) " << MirrorStart[2][0]/cm << "," << MirrorStart[2][2]/cm  << G4endl
     << "Mirror3 corner2 (x,z) " << MirrorEnd[2][0]/cm << "," << MirrorEnd[2][2]/cm << G4endl
     << "Mirror4 corner1 (x,z) " << MirrorStart[3][0]/cm  << "," << MirrorStart[3][2]/cm << G4endl
     << "Mirror4 corner2 (x,z) " << MirrorEnd[3][0]/cm  << "," << MirrorEnd[3][2]/cm  << G4endl
	 << "Mirror1 sizes, dx: " << MirrorWidth[0][0]/cm << " dy: " << MirrorWidth[0][1]/cm << " dz: " << MirrorWidth[0][2]/cm << G4endl
	 << "Mirror2 sizes, dx: " << MirrorWidth[1][0]/cm << " dy: " << MirrorWidth[1][1]/cm << " dz: " << MirrorWidth[1][2]/cm << G4endl
	 << "Mirror3 sizes, dx: " << MirrorWidth[2][0]/cm << " dy: " << MirrorWidth[2][1]/cm << " dz: " << MirrorWidth[2][2]/cm << G4endl
	 << "Mirror4 sizes, dx: " << MirrorWidth[3][0]/cm << " dy: " << MirrorWidth[3][1]/cm << " dz: " << MirrorWidth[3][2]/cm << G4endl
	 << "Mirror 1 tilt angle " << MirrorAngle[0] <<  G4endl
	 << "Mirror 2 tilt angle " << MirrorAngle[1] <<  G4endl
	 << "Mirror 3 tilt angle " << MirrorAngle[2] <<  G4endl
	 << "Mirror 4 tilt angle " << MirrorAngle[3] <<  G4endl
	 << "PMT 1 position, x: " << PMTPosition[0][0] << " z: " << PMTPosition[0][2] << G4endl
	 << "PMT 2 position, x: " << PMTPosition[1][0] << " z: " << PMTPosition[1][2] << G4endl
	 << "PMT 3 position, x: " << PMTPosition[2][0] << " z: " << PMTPosition[2][2] << G4endl
	 << "PMT 4 position, x: " << PMTPosition[3][0] << " z: " << PMTPosition[3][2] << G4endl
	 << "PMT radius:  " << PMT_R    << G4endl
	 << "SHMS central momentum: " << SHMS_P0 << G4endl 
	 << "Number of Planes :" << NumPlanes << G4endl
	 << "Distance between Planes: " << PlaneDistance << G4endl
	 << "Particle type: " << analysis->Particle << G4endl
	 << G4endl;
  G4cout << "-----------------------------------------------------" << G4endl;

  G4double CerLen = TankLength / 2.0*cm;

  for(int mirror=0; mirror<4; mirror++) {
	  	if((mirror == MIRROR_3)||(mirror == MIRROR_4)) {
			// We want these to be at a negative x
			MirrorPosition[mirror][X_AXIS] = -Mirror[mirror][X_AXIS] - MirrorWidth[mirror][Z_AXIS]*(sin (MirrorAngle[mirror] *degree)) + MirrorStart[mirror][X_AXIS];
		} else {
			MirrorPosition[mirror][X_AXIS] = Mirror[mirror][X_AXIS] - MirrorWidth[mirror][Z_AXIS]*(sin (MirrorAngle[mirror] *degree)) + MirrorStart[mirror][X_AXIS];
		}
	  	if((mirror == MIRROR_2)||(mirror == MIRROR_4)) {
			// We want these to be at a negative y
			MirrorPosition[mirror][Y_AXIS] = -Mirror[mirror][Y_AXIS];
		} else {
			MirrorPosition[mirror][Y_AXIS] = Mirror[mirror][Y_AXIS];
		}
 		MirrorPosition[mirror][Z_AXIS] = (MirrorEnd[mirror][Z_AXIS]-(MirrorEnd[mirror][Z_AXIS]-MirrorStart[mirror][Z_AXIS])/2.0) - CerLen + MirrorWidth[mirror][Z_AXIS]*(cos (MirrorAngle[mirror] *degree));
  }
  if(MirrorStart[0][2] != MirrorStart[1][2]) {
	  MirrorPosition[0][1] = MirrorPosition[0][1]-0.5*Overlap*cm;
	  MirrorPosition[1][1] = MirrorPosition[1][1]+0.5*Overlap*cm;
  }
  if(MirrorStart[2][2] != MirrorStart[3][2]) {
	  MirrorPosition[2][1] = MirrorPosition[2][1]-0.5*Overlap*cm;
	  MirrorPosition[3][1] = MirrorPosition[3][1]+0.5*Overlap*cm;
  }
  
  PMT_L  = 0.5*cm;

  char fname[100];
  char mirror;

  if(MirrorSphere) {
	  mirror = 's';
  } else {
	  mirror = 'p';
  }

  // We want to check if user wants binary or a text file

  char pmt_flag;
  
  if(PMTon) {
	  pmt_flag = 'y';
  } else {
	  pmt_flag = 'n';
  }
  
  if(Binary) {
  	sprintf(fname , "cer_R%d_AngUp%dAngDn%d_PMTUp%dPMTDn%d_P%.fM%cC%c.bin" , 
	  int(MirrorR) , (int(fabs(MirrorAngle[0]))) , (int(MirrorAngle[2])) ,int(PMTAngle[0]), int(PMTAngle[2]), SHMS_P0, mirror, pmt_flag);
	  analysis->Bin = true;
  } else {
  	sprintf(fname , "cer_R%d_AngUp%dAngDn%d_PMTUp%dPMTDn%d_P%.fM%cC%c.dat" , 
	  int(MirrorR) , (int(fabs(MirrorAngle[0]))) , (int(MirrorAngle[2])) ,int(PMTAngle[0]), int(PMTAngle[2]), SHMS_P0, mirror, pmt_flag);
	  analysis->Bin = false;
  }

  analysis->PMTon = PMTon;

  G4cout << fname << G4endl;
  analysis->fout_cer.open(fname,ios::out|ios::binary); // ios::binary allows for both text and binary output
  
  sprintf(fname , "hits_R%d_AngUp%dAngDn%d_PMTUp%dPMTDn%d_P%.fM%cC%c.dat" , 
  	  int(MirrorR) , (int(fabs(MirrorAngle[0]))) , (int(MirrorAngle[2])) ,int(PMTAngle[0]), int(PMTAngle[2]), SHMS_P0, mirror, pmt_flag);
  analysis->fout_hit.open(fname,ios::out|ios::binary); // ios::binary allows for both text and binary output

  if( !analysis->fout_cer.is_open() ) { G4cerr << "Can't open file " << fname << G4endl; exit(1); }
 
  // Output file header in plain text to pass information on the properties of the mirrors
  // and placement of the PMTs
  analysis->fout_cer 
	  		 << MirrorR << "  "
		     << MirrorAngle[0] << "  "
		     << MirrorAngle[2] << "  "
             << PMTAngle[0] << "  "
             << PMTAngle[1] << "  "
             << PMTAngle[2] << "  "
             << PMTAngle[3] << "  "
			 << PMT_R << "  "
			 << FrontOfTank << "  "
		     << G4endl;

  analysis->fout_cer 
	        << PMTPosition[0][0]/cm << "  " 
			<< PMTPosition[0][1]/cm << "  "  
			<< PMTPosition[0][2]/cm << "  "  
  			<< PMTPosition[1][0]/cm << "  " 
			<< PMTPosition[1][1]/cm << "  "  
			<< PMTPosition[1][2]/cm << "  "  
  			<< PMTPosition[2][0]/cm << "  " 
			<< PMTPosition[2][1]/cm << "  "  
			<< PMTPosition[2][2]/cm << "  "  
  			<< PMTPosition[3][0]/cm << "  " 
			<< PMTPosition[3][1]/cm << "  "  
			<< PMTPosition[3][2]/cm << "  "  
			<< G4endl;
  
  analysis->fout_cer 
		     << SHMS_P0 << "  "
	         << gasp << "  "
		     << G4endl;
  
  analysis->fout_hit 
	  		 << "Time(ns)" << "  "
		     << "World (x,y,z)" << "  "
		     << "PhotonID" << "  "
		     << G4endl;

}
