#include "TSHMSDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVParameterised.hh"

#include "G4UserLimits.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4ios.hh"

#include "G4NistManager.hh"
#include "G4Sphere.hh"
#include "G4OpticalSurface.hh"
#include "G4LogicalBorderSurface.hh"
#include "TSHMSCerenkov.hh"
#include "TSHMSMaterials.hh"
#include "TSHMSAnalysisManager.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
TSHMSDetectorConstruction::TSHMSDetectorConstruction()
:solidWorld(0),  logicWorld(0),  physiWorld(0),
 solidLayer(0),logicLayer(0),physiLayer(0), 
 LayerMater(0)
{
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
TSHMSDetectorConstruction::~TSHMSDetectorConstruction()
{
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
G4VPhysicalVolume* TSHMSDetectorConstruction::Construct()
{
  //------------------------------ 
  // World
  //------------------------------ 
  analysis = TSHMSAnalysisManager::getInstance();
  TSHMSMaterialManager* matManager = TSHMSMaterialManager::getInstance();
  G4Material* vacum = matManager->FindMaterial(G4String("vacum"));

  G4double HalfWorldLengthX = 2.0*0.8*150.0 *cm;
  G4double HalfWorldLengthY = 2.0*0.8*150.0 *cm;
  G4double HalfWorldLengthZ = 2500.0 *cm;
  
  solidWorld = new G4Box( "world" , HalfWorldLengthX , HalfWorldLengthY , HalfWorldLengthZ );
  logicWorld = new G4LogicalVolume( solidWorld , vacum , "World", 0 , 0 , 0 );
  
  //  Must place the World Physical volume unrotated at (0,0,0).
  // 
  physiWorld = new G4PVPlacement(0,               // no rotation
                                 G4ThreeVector(), // at (0,0,0)
                                 logicWorld,      // its logical volume
				 				 "World",         // its name
                                 0,               // its mother  volume
                                 false,           // no boolean operations
                                 0);              // copy number
				 
  //------------------------------
  // Tracker segments
  //------------------------------
  // 
  // An example of Parameterised volumes
  // dummy values for G4Box -- modified by parameterised volume

  analysis->SetTankOffset(); // will be used in PrimaryGeneratorAction

  G4double Z_position = 0.0;

  TSHMSCerenkov* DetCerenkov = new TSHMSCerenkov();

  Z_position = DetCerenkov->GetZposition();

  //  G4VPhysicalVolume* physDetCerenkov = 

  G4double TankPosition[3];

  // We want to set the tank position within the world relative to the transport coordinates
  TankPosition[0] = analysis->TankToTransportX(0.0);
  TankPosition[1] = analysis->TankToTransportY(0.0);
  TankPosition[2] = analysis->TankToTransportZ(Z_position);

  DetCerenkov->ConstructCerenkov(physiWorld , logicWorld , G4ThreeVector(TankPosition[0],TankPosition[1],TankPosition[2]));

// //--------- Visualization attributes -------------------------------------------

  G4VisAttributes* BoxVisAtt= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
  logicWorld  ->SetVisAttributes(BoxVisAtt);  
  
//   G4VisAttributes* ChamberVisAtt = new G4VisAttributes(G4Colour(1.0,1.0,0.0));
//   logicBlock->SetVisAttributes(ChamberVisAtt);

//   //  logicCalo->SetVisAttributes(new G4VisAttributes(G4Colour::Blue()));

// //   logicPMT->SetVisAttributes(new G4VisAttributes(G4Colour::Red()));
  
// //   logicalCatod->SetVisAttributes(new G4VisAttributes(G4Colour::Magenta()));

//   //  G4VisAttributes* TubsVisAtt = new G4VisAttributes(G4Colour(1.0,1.0,1.0));
//   G4VisAttributes* TubsVisAtt = new G4VisAttributes(G4Colour::Red());
//   logicTubs->SetVisAttributes(TubsVisAtt);


//--------- example of User Limits -------------------------------

  // below is an example of how to set tracking constraints in a given
  // logical volume(see also in N02PhysicsList how to setup the processes
  // G4StepLimiter or G4UserSpecialCuts).
    
  // Sets a max Step length in the tracker region, with G4StepLimiter
  //
  //  G4double maxStep = 0.5*ChamberWidth; 
  //  logicTracker->SetUserLimits(new G4UserLimits(maxStep));
  
  // Set additional contraints on the track, with G4UserSpecialCuts
  //
  // G4double maxLength = 2*fTrackerLength, maxTime = 0.1*ns, minEkin = 10*MeV;
  // logicTracker->SetUserLimits(new G4UserLimits(maxStep,maxLength,maxTime,
  //                                               minEkin));
  
  return physiWorld;
}
