#include "TSHMSEventAction.hh"

#include "G4Event.hh"
#include "G4EventManager.hh"
#include "G4HCofThisEvent.hh"
#include "G4VHitsCollection.hh"
#include "G4SDManager.hh"
#include "G4TrajectoryContainer.hh"
#include "G4Trajectory.hh"
#include "G4ios.hh"

#include "TSHMSAnalysisManager.hh"
#include "TSHMSPmtHit.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
TSHMSEventAction::TSHMSEventAction()
{
  G4String colName;
  G4SDManager* SDman = G4SDManager::GetSDMpointer();
  PHCID = SDman->GetCollectionID(colName="PmtSD/PmtCollection");
  G4cout << "Init TSHMSEventAction action" << G4endl;
  //  getchar();
}
TSHMSEventAction::TSHMSEventAction(TSHMSSteppingAction *st):StepAction(st)
{
  G4String colName;
  G4SDManager* SDman = G4SDManager::GetSDMpointer();
  PHCID = SDman->GetCollectionID(colName="PmtSD/PmtCollection");
  G4cout << "Init TSHMSEventAction action" << G4endl;
  //  getchar();
}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
TSHMSEventAction::~TSHMSEventAction()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
void TSHMSEventAction::BeginOfEventAction(const G4Event*)
{

  //  G4cout << " START OF A NEW EVENT " << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
void TSHMSEventAction::EndOfEventAction(const G4Event* evt)
{
  //G4cout << "EndOfEvent" << G4endl;
  G4HCofThisEvent* HCE = evt->GetHCofThisEvent();
  TSHMSPmtHitCollection* PHC = 0;

  if(HCE) {
	  PHC = (TSHMSPmtHitCollection*)(HCE->GetHC(PHCID));
  }
  if(PHC) {
	  G4int n_hit = PHC->entries();
	  G4cout << "PMT has " << n_hit << " hits." << G4endl;
	  for(int ii=0; ii<n_hit; ii++) {
	  	  TSHMSPmtHit* aHit = (*PHC)[ii];
      	  aHit->Print();
	  }
  }


  //  TSHMSAnalysisManager* analysis = TSHMSAnalysisManager::getInstance();
  
  G4int event_id = evt->GetEventID();
  //  if ( ( event_id % 1000 )==0 ) G4cout << "Event = " << event_id << G4endl;
  if ( ( event_id % 100 )==0 && ( event_id != 0)) G4cout << "Event = " << event_id << G4endl;
  
  // get number of stored trajectories
  //
  G4TrajectoryContainer* trajectoryContainer = evt->GetTrajectoryContainer();
  G4int n_trajectories = 0;
  if (trajectoryContainer) n_trajectories = trajectoryContainer->entries();
  
  // periodic printing
  //
//   if (event_id < 100 || event_id%100 == 0) {
//     G4cout << ">>> Event " << evt->GetEventID() << G4endl;
//     G4cout << "    " << n_trajectories 
// 	   << " trajectories stored in this event." << G4endl;
//   }

//  G4cout << "Particle code = " << evt->GetPrimaryVertex()->GetPrimary()->GetPDGcode() << G4endl;

 //  G4cout << StepAction->GetLayer1DE() << " "
// 	 << StepAction->GetLayer2DE() << " "
// 	 << StepAction->GetLayer3DE() << " "
// 	 << StepAction->GetLayer4DE() << G4endl;
  
//   G4cout << StepAction->GetCaloEtot() << G4endl;
  //  G4cout << " END OF AN EVENT " << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
