#include "G4VProcess.hh"
#include <fstream>
#include "G4ios.hh"
#include "G4UnitsTable.hh"
#include "TSHMSMaterials.hh"
#include "G4Step.hh"
#include "G4NistManager.hh"

using namespace std;

TSHMSMaterialManager* TSHMSMaterialManager::instance = 0;

TSHMSMaterialManager::TSHMSMaterialManager()
{
  G4cout << " A single copy of Material class has been created " << G4endl;
  BuildMaterials();  
}
//------------------------------------------------------------------------------
TSHMSMaterialManager::~TSHMSMaterialManager() 
{
  G4cout << "Destructor for materials maneger is called" << G4endl;
}
//------------------------------------------------------------------------------
TSHMSMaterialManager* TSHMSMaterialManager::getInstance()
{
  if ( instance == 0 ) instance = new TSHMSMaterialManager;
  G4cout << "Create Material manager " << G4endl;
  return instance;
}
//------------------------------------------------------------------------------
void  TSHMSMaterialManager::BuildMaterials()
{
  G4NistManager* man = G4NistManager::Instance();

  G4double a, z;
  G4double density;//, temperature, pressure;
  G4int nel;

  G4Material* vacum       = man->FindOrBuildMaterial("G4_Galactic");         // vacuum
  fMaterialMap[G4String("vacum")]=vacum;

  G4Material* LeadOxide   = man->FindOrBuildMaterial("G4_LEAD_OXIDE");       // PbO
  G4Material* SilicOxide  = man->FindOrBuildMaterial("G4_SILICON_DIOXIDE");  // SiO2
  G4Material* PotasOxide  = man->FindOrBuildMaterial("G4_POTASSIUM_OXIDE");  // K2O
  G4Material* SodMonOxide = man->FindOrBuildMaterial("G4_SODIUM_MONOXIDE");  // Na2O

  G4Element* As = new G4Element( "Arsen"   ,  "As", z=33.0 , a= 74.92  *g/mole );
  G4Element* O  = new G4Element( "Oxygen"  ,  "O",  z=8.0  , a= 16.00  *g/mole );
  G4Element* N  = new G4Element( "Nitrogen",  "N",  z=7.0  , a= 14.01  *g/mole );
  G4Element* Sb = new G4Element( "Antimony",  "Sb", z=51.0 , a= 121.76 *g/mole );
  G4Element* Cs = new G4Element( "Cesium"  ,  "Cs", z=55.0 , a= 132.91 *g/mole );
  G4Element* K  = new G4Element( "Potassium", "K",  z=19.0 , a= 39.0983*g/mole );
  G4Element* H  = new G4Element( "Hidrogen",  "H",  z=1.0  , a= 1.00   *g/mole );
  G4Element* F  = new G4Element( "Fluorine",  "F",  z=9.0  , a= 19.00  *g/mole );
  G4Element* C  = new G4Element( "Carbon"  ,  "C",  z=6.0  , a= 12.01  *g/mole );
  //  G4Element* Ne = new G4Element( "Neon"    ,  "Ne", z=10.0 , a= 20.1797*g/mole );
  
  G4Material* GEBialkali = new G4Material("GEBA", density= 3.0*g/cm3, nel=3 ); // green extended bi-alkali // cathode for PMT XP3462B
  GEBialkali->AddElement( K  , 1 );
  GEBialkali->AddElement( Cs , 1 );
  GEBialkali->AddElement( Sb , 1 );

  fMaterialMap[G4String("GEBialkali")]=GEBialkali;

  G4Material* Tedlar = new G4Material("Tedlar", density= 1.42 *g/cm3, nel=3 ); // Tedlar
  Tedlar->AddElement( H , 3 );
  Tedlar->AddElement( C , 2 );
  Tedlar->AddElement( F , 1 );

  fMaterialMap[G4String("Tedlar")]=Tedlar;
  
  G4Material* As2O3 = new G4Material("As2O3", density= 3.738*g/cm3, nel=2); // As2O3

  As2O3->AddElement( As, 2 );
  As2O3->AddElement( O , 3 );

  G4Material* LgTF1 = new G4Material("LgTF1", density= 3.86*g/cm3 , nel=5 );

  LgTF1->AddMaterial( LeadOxide   , 0.5120 );
  LgTF1->AddMaterial( SilicOxide  , 0.4130 );
  LgTF1->AddMaterial( PotasOxide  , 0.0422 );
  LgTF1->AddMaterial( SodMonOxide , 0.0278 );
  LgTF1->AddMaterial( As2O3       , 0.0050 );

//
// ------------ Generate & Add Material Properties Table ------------
//
  const G4int nEntries = 15;

  G4double PhotonEnergyTF1[nEntries] =
            { 1.99988723*eV, 2.06655002*eV, 2.13781047*eV,  
	      2.21416092*eV, 2.29616666*eV, 2.38448095*eV,
              2.47986007*eV, 2.58318758*eV, 2.69550014*eV,
	      2.81802297*eV, 2.95221448*eV, 3.09982514*eV,
	      3.26297379*eV, 3.44425011*eV, 3.64685321*eV };

  G4double RefractiveIndex1[nEntries] =
    { 1.6436, 1.6454, 1.6474, 1.6496, 1.6521, 
      1.6548, 1.6579, 1.6614, 1.6653, 1.6697, 
      1.6748, 1.6806, 1.6874, 1.6952, 1.7045 };

  G4double Absorption1[nEntries] =
           { 7.0*m, 7.0*m, 7.0*m, 7.0 *m, 7.0 *m,
	     7.0*m, 6.0*m, 5.0*m, 3.5 *m, 2.75*m,
	     1.8*m, 1.0*m, 0.5*m, 0.15*m, 0.08*m };

  G4MaterialPropertiesTable* TF1glass = new G4MaterialPropertiesTable();
  TF1glass->AddProperty("RINDEX",       PhotonEnergyTF1, RefractiveIndex1, nEntries);
  TF1glass->AddProperty("ABSLENGTH",    PhotonEnergyTF1, Absorption1,      nEntries);
  
  LgTF1->SetMaterialPropertiesTable(TF1glass);

  fMaterialMap[G4String("LgTF1")]=LgTF1;

  //-------------------------------------------------------------------------------

  // -- Material C4F10 Perfluorobutane

  G4Material* Perfluorbutane = 
    new G4Material ("Perfluorobutane",density=9.935*mg/cm3, 2 ,kStateGas ,293.15*kelvin, 0.6*atmosphere );
  Perfluorbutane->AddElement( C , 4 );
  Perfluorbutane->AddElement( F , 10 );

  fMaterialMap[G4String("Perfluorobutane")]=Perfluorbutane;

  const G4int nEntriesC4F10 = 5;

  G4double PhotonEnergyC4F10[nEntriesC4F10] = { 2.0*eV, 3.0*eV, 4.0*eV, 5.0*eV, 6.0*eV };
  G4double RefractiveIndexC4F10[nEntriesC4F10] = { 1.0006, 1.0006, 1.0006, 1.0006, 1.0006 };
  G4double AbsorptionC4F10[nEntriesC4F10] = { 100.0 *m , 100.0 *m , 100.0 *m, 100.0 *m , 100.0 *m };
  G4MaterialPropertiesTable* matC4F10 = new G4MaterialPropertiesTable();
  matC4F10->AddProperty("RINDEX", PhotonEnergyC4F10    , RefractiveIndexC4F10 , nEntriesC4F10 );
  matC4F10->AddProperty("ABSLENGTH", PhotonEnergyC4F10 , AbsorptionC4F10      , nEntriesC4F10 );
  Perfluorbutane->SetMaterialPropertiesTable(matC4F10);

  // -- End of Perfluorobutane

  // -- Material C4F8O, OctaFluoroTetraHydroFuran
  
  G4double C4F8O_pressure = 0.0;
  string line;
  ifstream fin("cer_config.dat" , ios::in );
  if( !fin.is_open() ) { G4cerr << "Can't open file cer_config.dat " << G4endl; exit(1);}

  while( !fin.eof()){

    getline( fin , line );

    string::size_type loc = line.find( "Gas pressure:" , 0 );

    if( loc != string::npos ) {
      string::size_type loc_beg = line.find(":",0);
      string::size_type loc_end = line.size();

      string aa = line.substr( loc_beg + 1 , ( loc_end - loc_beg ) );
      C4F8O_pressure = atof( aa.c_str() );
      break;
    }
  }
  fin.close();
  if( C4F8O_pressure <= 0.0 ) { G4cerr << "Cerenkov gas pressure is wrong: " << C4F8O_pressure << " atmosphere" << G4endl; exit(1); }
  else                        { G4cerr << "Cerenkov gas pressure is : " << C4F8O_pressure << " atmosphere" << G4endl; };

  G4Material* C4F8O = new G4Material ("C4F8O",density=8.46117*mg/cm3, 3 ,kStateGas ,293.15*kelvin, C4F8O_pressure*atmosphere );

 C4F8O->AddElement( C , 4 );
 C4F8O->AddElement( F , 8 );
 C4F8O->AddElement( O , 1 );

 fMaterialMap[G4String("C4F8O")]=C4F8O;

 const G4int nEntriesC4F8O = 5;

 G4double PhotonEnergyC4F8O[nEntriesC4F8O] = { 2.0*eV, 3.0*eV, 4.0*eV, 5.0*eV, 6.0*eV };
 G4double RefractiveIndexC4F8O[nEntriesC4F8O];// = { 1.00137,1.00137,1.00137,1.00137,1.00137 };
 for( int ii = 0 ; ii < nEntriesC4F8O; ii++ ) RefractiveIndexC4F8O[ii] = 1.0 + 0.00137*C4F8O_pressure;
 G4double AbsorptionC4F8O[nEntriesC4F8O] = { 100.0 *m , 100.0 *m , 100.0 *m, 100.0 *m , 100.0 *m };
 G4MaterialPropertiesTable* matC4F8O = new G4MaterialPropertiesTable();
 matC4F8O->AddProperty("RINDEX", PhotonEnergyC4F8O    , RefractiveIndexC4F8O , nEntriesC4F8O );
 matC4F8O->AddProperty("ABSLENGTH", PhotonEnergyC4F8O , AbsorptionC4F8O      , nEntriesC4F8O );
 C4F8O->SetMaterialPropertiesTable(matC4F8O);

  // -- End of OctaFluoroTetraHydroFuran

  // -- Material MgF
 //  G4Element* Mg = new G4Element("Magnezium", "Mg", z=12.0 , a=24.305 *g/mole);

  // -- End of MgF  

  G4Material* Mylar = new G4Material ("Mylar",density=1.39*g/cm3, 3 );
  Mylar->AddElement( C , 5.0 );
  Mylar->AddElement( H , 4.0 );
  Mylar->AddElement( O , 2.0 );

  G4Element* Al = new G4Element("Aluminum",  "Al" , z = 13.0, a = 26.98*g/mole ); 
  G4Material* AlMirror = man->FindOrBuildMaterial("G4_Al"); // For Cerenkov mirror
  fMaterialMap[G4String("AlMirror")] = AlMirror;
  
  G4Material* AlTank = man->FindOrBuildMaterial("G4_Al"); // For Cerenkov mirror
  fMaterialMap[G4String("AlTank")] = AlTank;

  G4Material* AlMylar = new G4Material ("AlMylar", density = 1.0*g/cm3, 4 );
  AlMylar->AddElement( C , 0.450145 );
  AlMylar->AddElement( H , 0.030220 );
  AlMylar->AddElement( O , 0.239842 );
  AlMylar->AddElement( Al ,0.279793 );

  G4double RefractiveIndexMylar[nEntries] =
    { 0.0, 0.0, 0.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.0, 0.0 };
  G4double AbsorptionMylar[nEntries] =
    { 1.0, 1.0, 1.0, 1.0, 1.0,
      1.0, 1.0, 1.0, 1.0, 1.0,
      1.0, 1.0, 1.0, 1.0, 1.0 };
  G4double ReflectivityMylar[nEntries] =
    { 1.0, 1.0, 1.0, 1.0, 1.0,
      1.0, 1.0, 1.0, 1.0, 1.0,
      1.0, 1.0, 1.0, 1.0, 1.0 };

  G4MaterialPropertiesTable* prMylar = new G4MaterialPropertiesTable();
  prMylar->AddProperty("RINDEX",       PhotonEnergyTF1, RefractiveIndexMylar ,nEntries);
  prMylar->AddProperty("ABSLENGTH",    PhotonEnergyTF1, AbsorptionMylar      ,nEntries);
  prMylar->AddProperty("REFLECTIVITY", PhotonEnergyTF1, ReflectivityMylar    ,nEntries);

  AlMylar->SetMaterialPropertiesTable(prMylar);
  fMaterialMap[G4String("AlMylar")]=AlMylar;

  
  
  // -- Cathode
  G4Material* AlCathode = new G4Material("AlCathode", density=2.7*g/cm3, 1);
  AlCathode->AddElement(Al, 1.0);

  G4double ppCathode[2] = {1.60*eV, 13.00*eV};
  G4double effCathode[2] = {0.3, 0.3};
  G4double reflCathode[2] = {0., 0.};

  G4MaterialPropertiesTable* prAlCathode = new G4MaterialPropertiesTable();
  prMylar->AddProperty("EFFICIENCY", ppCathode, effCathode ,2);
  prMylar->AddProperty("REFLECTIVITY", ppCathode, reflCathode ,2);

  AlCathode->SetMaterialPropertiesTable(prAlCathode);
  fMaterialMap[G4String("AlCathode")]=AlCathode;



  // -- Air optical
  
  G4Material* Air = new G4Material("Air", density=1.29*mg/cm3, 2 );
  Air->AddElement(N, 70.*perCent);
  Air->AddElement(O, 30.*perCent);

  G4double RefractiveIndexAir[nEntries] =
    { 1.00, 1.00, 1.00, 1.00, 1.00,
      1.00, 1.00, 1.00, 1.00, 1.00,
      1.00, 1.00, 1.00, 1.00, 1.00 };
  G4double ReflectivityAir[nEntries] =
    { 0.0, 0.0, 0.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.0, 0.0 };
  G4double AbsorptionAir[nEntries]=
    { 100.0 *m, 100.0 *m, 100.0 *m, 100.0 *m, 100.0 *m, 
      100.0 *m, 100.0 *m, 100.0 *m, 100.0 *m, 100.0 *m, 
      100.0 *m, 100.0 *m, 100.0 *m, 100.0 *m, 100.0 *m };

  G4MaterialPropertiesTable* AirOp = new G4MaterialPropertiesTable();
  AirOp->AddProperty("RINDEX",       PhotonEnergyTF1 , RefractiveIndexAir  , nEntries);
  AirOp->AddProperty("REFLECTIVITY", PhotonEnergyTF1 , ReflectivityAir  ,    nEntries );
  AirOp->AddProperty("ABSLENGTH",    PhotonEnergyTF1 , AbsorptionAir  ,      nEntries );

  //  Air->SetMaterialPropertiesTable(AirOp);
  fMaterialMap[G4String("Air")]=Air;

  // -- Soda lime glass

  G4Material* BoronOxide = man->FindOrBuildMaterial("G4_BORON_OXIDE"); // B2O3
  G4Material* AlumnOxide = man->FindOrBuildMaterial("G4_ALUMINUM_OXIDE"); // Al2O3
  G4Material* CalciumOxide = man->FindOrBuildMaterial("G4_CALCIUM_OXIDE"); // CaO
  G4Material* MagnesiumOxide = man->FindOrBuildMaterial("G4_MAGNESIUM_OXIDE"); // MgO

  G4Element* Ba = new G4Element("Barium", "Ba", z = 56.0 , a= 137.327*g/mole);
  G4Element* Ir = new G4Element("Iron", "Ir", z = 26.0 , a= 55.8457*g/mole);

  G4Material* BariumOxide = new G4Material ("BariumOx", density = 5.7 * g/cm3, 2 );
  BariumOxide->AddElement( Ba , 1 );  
  BariumOxide->AddElement( O , 1 );  
  G4Material* IronOxide = new G4Material ("IronOx", density = 5.24 * g/cm3, 2 );
  IronOxide->AddElement( Ir , 2 );
  IronOxide->AddElement( O  , 3 );

  G4Material* LimeGlass = new G4Material ("LimeGlass", density = 2.47 * g/cm3, 9 );
  LimeGlass->AddMaterial( SilicOxide ,   0.723 );
  LimeGlass->AddMaterial( BoronOxide ,   0.003 );
  LimeGlass->AddMaterial( AlumnOxide ,   0.027 );
  LimeGlass->AddMaterial( CalciumOxide , 0.103 );
  LimeGlass->AddMaterial( MagnesiumOxide,0.003 );
  LimeGlass->AddMaterial( SodMonOxide,   0.134 );
  LimeGlass->AddMaterial( PotasOxide ,   0.005 );
  LimeGlass->AddMaterial( BariumOxide ,  0.002 );
  LimeGlass->AddMaterial( IronOxide ,    0.0004);

  G4double RefractiveIndexLimeGlass[]=
    { 1.512, 1.512, 1.512, 1.512, 1.512, 
      1.512, 1.512, 1.512, 1.512, 1.512, 
      1.512, 1.540, 1.512, 1.512, 1.512 };
  G4double ReflectivityLimeGlass[]=
    { 1.0-0.786, 1.0-0.850, 1.0-0.9 , 1.0-0.915, 1.0-0.910,
      1.0-0.91 , 1.0-0.91 , 1.0-0.91, 1.0-0.920, 1.0-0.920,
      1.0-0.92 , 1.0-0.92 , 1.0-0.91,  1.0-0.91, 1.0-0.905 };
  G4double AbsorptionLimeGlass[]=
    { 1.0 *m, 1.0 *m, 1.0 *m, 1.0 *m, 1.0 *m, // temporary
      1.0 *m, 1.0 *m, 1.0 *m, 1.0 *m, 1.0 *m, 
      1.0 *m, 1.0 *m, 1.0 *m, 1.0 *m, 1.0 *m };

  G4MaterialPropertiesTable* PMTglass = new G4MaterialPropertiesTable();
  PMTglass->AddProperty("RINDEX",       PhotonEnergyTF1 , RefractiveIndexLimeGlass , nEntries );
  PMTglass->AddProperty("REFLECTIVITY", PhotonEnergyTF1 , ReflectivityLimeGlass ,    nEntries );
  PMTglass->AddProperty("ABSLENGTH",    PhotonEnergyTF1 , AbsorptionLimeGlass ,      nEntries );
  
  LimeGlass->SetMaterialPropertiesTable( PMTglass );
  
  fMaterialMap[G4String("LimeGlass")]=LimeGlass;

  // -- End Soda lime glass

  // -- Corning 9741 UV glass

  G4Material* LiOxide = man->FindOrBuildMaterial("G4_LITHIUM_OXIDE"); // Li2O
  G4Material* UVGlass = new G4Material ("UVGlass",density=2.16 *g/cm3, 5 );  
  UVGlass->AddMaterial( SilicOxide , 0.65 );
  UVGlass->AddMaterial( BoronOxide , 0.27 );
  UVGlass->AddMaterial( AlumnOxide , 0.05 );
  UVGlass->AddMaterial( LiOxide    , 0.01 );
  UVGlass->AddMaterial( SodMonOxide, 0.02 );

  //G4double RefractIndexUVglass[5] = { 1.468 , 1.468 , 1.468 , 1.468 , 1.468 };
  G4double AbsorptionUVGlass  [5] = { 10.0 *m , 10.0 *m , 10.0 *m , 10.0 *m , 10.0 *m };
 /*
  G4MaterialPropertiesTable* UVGlassOpTb = new G4MaterialPropertiesTable();

  UVGlassOpTb->AddProperty("RINDEX",    PhotonEnergyC4F10 , RefractIndexUVglass , nEntriesC4F10 );
  UVGlassOpTb->AddProperty("ABSLENGTH", PhotonEnergyC4F10 , AbsorptionUVGlass   , nEntriesC4F10 );
 
  */
  
  const G4int NUM = 14;
  G4double pp[NUM] = { 1.653*eV, 4.133*eV, 4.275*eV, 4.428*eV, 4.592*eV, 4.769*eV, 4.959*eV, 5.061*eV, 5.209*eV, 5.391*eV, 5.767*eV, 6.525*eV, 6.888*eV, 12.398*eV };
  //G4double pp[NUM] = { 750*nm, 300*nm, 290*nm, 280*nm, 270*nm, 260*nm, 250*nm, 245*nm, 238*nm, 230*nm, 215*nm, 190*nm, 180*nm, 100*nm }; 
  G4double eff[NUM] = { 0.0,  0.0,  0.1,  0.55,  0.68,  0.66,  0.64,  0.68,  0.78,  0.84,  0.88,  0.91,  0.92,  0.92 };
  G4double ref[NUM] = { 1.0,  1.0,  0.9,  0.45,  0.32,  0.34,  0.36,  0.32,  0.22,  0.16,  0.12,  0.09,  0.08,  0.08 };
  G4double rindex[NUM] = { 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585 };
 /* 
  const G4int NUM = 24;
  G4double pp[NUM] = {  1.653*eV, 1.771*eV, 1.968*eV, 2.066*eV, 2.254*eV, 2.480*eV, 2.695*eV, 3.100*eV, 
	  					4.133*eV, 4.275*eV, 4.428*eV, 4.592*eV, 4.769*eV, 4.959*eV, 5.061*eV, 5.209*eV, 
						5.391*eV, 5.767*eV, 6.199*eV, 6.525*eV, 6.888*eV, 7.085*eV, 7.749*eV, 12.398*eV };
  G4double trans[NUM] = { 0.000, 0.000, 0.002, 0.009, 0.063, 0.138, 0.184, 0.226, 0.200, 0.197, 0.191, 0.192, 
	  					  0.169, 0.148, 0.139, 0.143, 0.148, 0.119, 0.022, 0.022, 0.000, 0.000, 0.000, 0.000 };
  G4double refl[NUM] = {1.000, 1.000, 0.998, 0.991, 0.937, 0.862, 0.816, 0.774, 0.800, 0.803, 0.809, 0.808,
	  					0.831, 0.852, 0.861, 0.857, 0.852, 0.881, 0.978, 0.978, 1.000, 1.000, 1.000, 1.000};
  G4double rindex[NUM] = { 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 
  	                       1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585, 1.4585 };
*/
  G4MaterialPropertiesTable *UVGlassOpTb = new G4MaterialPropertiesTable();
  UVGlassOpTb->AddProperty("RINDEX"               , pp , rindex  , NUM );
  UVGlassOpTb->AddProperty("REFLECTIVITY"         , pp , ref  , NUM );
  UVGlassOpTb->AddProperty("EFFICIENCY"           , pp , eff   , NUM );
  UVGlassOpTb->AddProperty("ABSLENGTH", PhotonEnergyC4F10 , AbsorptionUVGlass   , nEntriesC4F10 );

  UVGlass->SetMaterialPropertiesTable( UVGlassOpTb );
  
  fMaterialMap[G4String("UVGlass")]=UVGlass;

  // -- End Corning 9741 UV glass

  
  // -- Bialkali photocathode K2CsSb 

  G4Material* BialkaliK2CsSb = new G4Material ("BialkaliK2CsSb" , density= 2.9 *g/cm3 , 3 ); // density need to be checked later 
  
  BialkaliK2CsSb->AddElement( K  , 2 );
  BialkaliK2CsSb->AddElement( Cs , 1 );
  BialkaliK2CsSb->AddElement( Sb , 1 );

  G4double K2CsSbCathLen = 0.00004 * mm;
  G4double K2CsSbNominalTransmision = 0.5;
  G4double K2CsSbCathAbsorpLen      =  -(K2CsSbCathLen)/(std::log(K2CsSbNominalTransmision));

  G4double RefractIndexK2CsSb[5]={ 1.4 , 1.4 , 1.4 , 1.4 , 1.4 };
  G4double AbsorptionK2CsSb[5]  =
    { K2CsSbCathAbsorpLen , K2CsSbCathAbsorpLen , K2CsSbCathAbsorpLen , K2CsSbCathAbsorpLen , K2CsSbCathAbsorpLen };
  
  G4MaterialPropertiesTable* K2CsSbOpTb = new G4MaterialPropertiesTable();

  K2CsSbOpTb->AddProperty("RINDEX",    PhotonEnergyC4F10 , RefractIndexK2CsSb , nEntriesC4F10 );
  K2CsSbOpTb->AddProperty("ABSLENGTH", PhotonEnergyC4F10 , AbsorptionK2CsSb   , nEntriesC4F10 );
  
  BialkaliK2CsSb->SetMaterialPropertiesTable( K2CsSbOpTb );

  fMaterialMap[G4String("BialkaliK2CsSb")]=BialkaliK2CsSb;

  // -- Polyvinyltoluene , sccintilator for HMS hodoscope
  G4Material* Polyvinyltoluene = man->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");  // [CH2CH(C6H4CH3)]
  fMaterialMap[G4String("Polyvinyltoluene")]=Polyvinyltoluene;

  const G4int ScinEnBins = 32;

  G4double ScinEnergies[ScinEnBins]=
    { 2.3845 *eV, 2.4076 *eV, 2.4312 *eV, 2.4553 *eV, 2.4799 *eV, 2.5049 *eV, 2.5305 *eV, 2.5566 *eV,
      2.5832 *eV, 2.6104 *eV, 2.6381 *eV, 2.6665 *eV, 2.6955 *eV, 2.7251 *eV, 2.7554 *eV, 2.7864 *eV,
      2.8180 *eV, 2.8504 *eV, 2.8836 *eV, 2.9175 *eV, 2.9522 *eV, 2.9878 *eV, 3.0242 *eV, 3.0616 *eV,
      3.0998 *eV, 3.1391 *eV, 3.1793 *eV, 3.2206 *eV, 3.2630 *eV, 3.3065 *eV, 3.3512 *eV, 3.4000 *eV }; // 370-520 nm

  G4double ScinRefracIndex[ScinEnBins]=
    { 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58,
      1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58,
      1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58,
      1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58 };

  G4double ScinAbsorptionL[ScinEnBins] =
    { 210*cm, 210*cm, 210*cm, 210*cm, 210*cm,210*cm, 210*cm, 210*cm, 
      210*cm, 210*cm, 210*cm, 210*cm, 210*cm,210*cm, 210*cm, 210*cm, 
      210*cm, 210*cm, 210*cm, 210*cm, 210*cm,210*cm, 210*cm, 210*cm, 
      210*cm, 210*cm, 210*cm, 210*cm, 210*cm,210*cm, 210*cm, 210*cm };

  G4double Scnt_FAST[ScinEnBins] = 
    { 0.00576, 0.00743, 0.00958, 0.01236, 0.01594, 0.02056, 0.02652, 0.03421,
      0.04413, 0.05692, 0.07342, 0.09471, 0.12217, 0.15758, 0.20327, 0.28206,
      0.34800, 0.40777, 0.46135, 0.50877, 0.55000, 0.72082, 0.92363, 0.98746,
      0.88082, 0.59623, 0.31661, 0.13368, 0.03294, 0.00130, 0.0    , 0.0 };

  //  G4double Scnt_SLOW[ScinEnBins] = { 0.00001,0.00001,0.00001,0.00001,0.00001,0.00001,0.00001,0.00001,0.00001,0.00001,
  //				     0.00001,0.00001,0.00001,0.00001,0.00001,0.00001,0.00001 };

  G4MaterialPropertiesTable* Scnt_MPT = new G4MaterialPropertiesTable();

  Scnt_MPT->AddProperty("RINDEX"       , ScinEnergies , ScinRefracIndex , ScinEnBins );
  Scnt_MPT->AddProperty("ABSLENGTH"    , ScinEnergies , ScinAbsorptionL , ScinEnBins );
  Scnt_MPT->AddProperty("FASTCOMPONENT", ScinEnergies , Scnt_FAST       , ScinEnBins );
  //  Scnt_MPT->AddProperty("SLOWCOMPONENT", ScinEnergies , Scnt_SLOW , ScinEnBins );

  Scnt_MPT->AddConstProperty("SCINTILLATIONYIELD", 500./MeV);
  Scnt_MPT->AddConstProperty("RESOLUTIONSCALE", 2.0);
  Scnt_MPT->AddConstProperty("FASTTIMECONSTANT",  1.*ns);
  //  Scnt_MPT->AddConstProperty("SLOWTIMECONSTANT", 10.*ns);
  Scnt_MPT->AddConstProperty("YIELDRATIO", 0.8);

  Polyvinyltoluene->SetMaterialPropertiesTable(Scnt_MPT);

  // -- Borosilcate glass, for Photonis XP2260 used in hodoscopes

  G4Material* BoroSilicateGlass = new G4Material ("BoroSilicateGlass", density = 2.23 * g/cm3, 4 );

  BoroSilicateGlass->AddMaterial( SilicOxide  , 0.807 ); // SiO2
  BoroSilicateGlass->AddMaterial( BoronOxide  , 0.130 ); // B2O3
  BoroSilicateGlass->AddMaterial( SodMonOxide , 0.040 ); // Na2O
  BoroSilicateGlass->AddMaterial( AlumnOxide  , 0.023 ); // Al2O3
  
  G4double RefracIndexBoroSilicateGlass[ScinEnBins]=
    { 1.4769, 1.4772, 1.4775, 1.4777, 1.4780, 1.4783, 1.4786, 1.4789, 
      1.4793, 1.4796, 1.4799, 1.4803, 1.4807, 1.4810, 1.4814, 1.4818, 
      1.4822, 1.4826, 1.4831, 1.4835, 1.4840, 1.4845, 1.4850, 1.4855, 
      1.4860, 1.4865, 1.4871, 1.4877, 1.4883, 1.4889, 1.4895, 1.4902 };

  G4double AbsorptionLBoroSilicateGlass[ScinEnBins]=
    { 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm,
      200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm,
      200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm,
      200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm, 200.0 *cm };

  G4double ReflectivityBoroSilicateGlass[ScinEnBins]=
    { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 };
  
  G4MaterialPropertiesTable* BorSiGlassMPT = new G4MaterialPropertiesTable();

  BorSiGlassMPT->AddProperty("RINDEX",       ScinEnergies , RefracIndexBoroSilicateGlass  , ScinEnBins );
  BorSiGlassMPT->AddProperty("REFLECTIVITY", ScinEnergies , ReflectivityBoroSilicateGlass , ScinEnBins );
  BorSiGlassMPT->AddProperty("ABSLENGTH",    ScinEnergies , AbsorptionLBoroSilicateGlass  , ScinEnBins );
  

  BoroSilicateGlass->SetMaterialPropertiesTable( BorSiGlassMPT );
  fMaterialMap[G4String("BoroSilicateGlass")]=BoroSilicateGlass;

  // -- End Borosilcate glass,

  // -- Lucite
  
  G4Material* Lucite =  new G4Material("Lucite", density= 1.185 * g/cm3 , 3 );
  Lucite->AddElement( C , 59.97 *perCent );
  Lucite->AddElement( H , 8.07  *perCent );
  Lucite->AddElement( O , 31.96 *perCent );

  fMaterialMap[G4String("Lucite")]=Lucite;

  const G4int LuciteEnBins = 17;

  G4double LuciteEnergies[LuciteEnBins] =
    { 2.38448 *eV, 2.43124 *eV, 2.47986 *eV, 2.53047 *eV, 2.58319 *eV, 2.63815 *eV, 2.69550 *eV, 2.75540 *eV, 2.81802 *eV, 
      2.88356 *eV, 2.95221 *eV, 3.02422 *eV, 3.09983 *eV, 3.17931 *eV, 3.26297 *eV, 3.35116 *eV, 3.44425 *eV }; // 520-360 nm

  G4double RefracIndexLucite[LuciteEnBins]=
    { 1.49491, 1.49569, 1.4965, 1.49735, 1.49824, 1.49917, 1.50014, 1.50114, 1.50219, 
      1.50327, 1.50439, 1.50554, 1.50674, 1.50797, 1.50924, 1.51055, 1.5119 };
  G4double LuciteAbsorptionL[LuciteEnBins]=
    { 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 
      20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m, 20.0 *m };
  G4double ReflectivityLucite[LuciteEnBins]=
    { 1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0, 
      1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0 };
  //    { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 
  //      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 };

  // -- Lucite optical properties

  G4MaterialPropertiesTable* Lucite_MPT = new G4MaterialPropertiesTable();

  Lucite_MPT->AddProperty("RINDEX"       , LuciteEnergies , RefracIndexLucite  , LuciteEnBins );
  Lucite_MPT->AddProperty("ABSLENGTH"    , LuciteEnergies , LuciteAbsorptionL  , LuciteEnBins );
  Lucite_MPT->AddProperty("REFLECTIVITY" , LuciteEnergies , ReflectivityLucite , LuciteEnBins );

  Lucite->SetMaterialPropertiesTable(Lucite_MPT);

  // -- End bialkali photocathode K2CsSb 

  const G4int AlMylarEnergyBins = 40;

  // -- Mylar energy bins 
  G4double PhotonEnergyAlMylar[AlMylarEnergyBins] =
    { 1.8001 *eV, 1.8667 *eV, 1.9333 *eV, 1.9999 *eV, 2.0665 *eV, 2.1378 *eV, 2.2142 *eV, 2.2962 *eV,
      2.3845 *eV, 2.4076 *eV, 2.4312 *eV, 2.4553 *eV, 2.4799 *eV, 2.5049 *eV, 2.5305 *eV, 2.5566 *eV,
      2.5832 *eV, 2.6104 *eV, 2.6381 *eV, 2.6665 *eV, 2.6955 *eV, 2.7251 *eV, 2.7554 *eV, 2.7864 *eV,
      2.8180 *eV, 2.8504 *eV, 2.8836 *eV, 2.9175 *eV, 2.9522 *eV, 2.9878 *eV, 3.0242 *eV, 3.0616 *eV,
      3.0998 *eV, 3.1391 *eV, 3.1793 *eV, 3.2206 *eV, 3.2630 *eV, 3.3065 *eV, 3.3512 *eV, 4.0000 *eV }; // 370-520 nm

  G4double ReflectivityAlMylar[AlMylarEnergyBins] =
    { 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 
      0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 
      0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 
      0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9, 0.9 };

  G4OpticalSurface *OpticalAlMylar = new G4OpticalSurface("OpticalAlMylar");

  OpticalAlMylar->SetType(dielectric_metal);
  OpticalAlMylar->SetModel(glisur);
  OpticalAlMylar->SetFinish(polished);

  G4MaterialPropertiesTable *AlMylarLayer = new G4MaterialPropertiesTable();
  AlMylarLayer->AddProperty("REFLECTIVITY", PhotonEnergyAlMylar , ReflectivityAlMylar , AlMylarEnergyBins );
  AlMylarLayer->AddProperty("EFFICIENCY"  , PhotonEnergyAlMylar , ReflectivityAlMylar , AlMylarEnergyBins );

  OpticalAlMylar->SetMaterialPropertiesTable(AlMylarLayer);

  fOpticalSurfaceMap[G4String("OpticalAlMylar")]=OpticalAlMylar;

  // -- Optical surface between volumes
  
  

  // -- End optical surface between volumes


  // Print all the materials defined.

  G4cout << G4endl << "The materials defined are : " << G4endl << G4endl; 
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;  
}
