#include "TSHMSPmtSD.hh"
#include "TSHMSPmtHit.hh"
#include "G4HCofThisEvent.hh"
#include "G4TouchableHistory.hh"
#include "G4Track.hh"
#include "G4Step.hh"
#include "G4ThreeVector.hh"
#include "G4SDManager.hh"
#include "G4ios.hh"
#include "G4VProcess.hh"

#include "G4OpBoundaryProcess.hh"
#include "G4ProcessManager.hh"

//#include "CLHEP/Random/RandFlat.h"
//-----------------------------------------------------------------------------------

TSHMSPmtSD::TSHMSPmtSD(G4String name)
:G4VSensitiveDetector(name)
{
  G4String HCname;
  collectionName.insert(HCname="PmtCollection");
  HCID = -1;

  analysisManager = TSHMSAnalysisManager::getInstance();
  G4cout << " TSHMSPmtSD Sensitive Detector Constructor " << HCname << G4endl;
}

//-----------------------------------------------------------------------------------

TSHMSPmtSD::~TSHMSPmtSD(){ }

//-----------------------------------------------------------------------------------

void TSHMSPmtSD::Initialize(G4HCofThisEvent* HCE)
{
   hitsCollection = new TSHMSPmtHitCollection
                           (SensitiveDetectorName,collectionName[0]); 
   if(HCID<0)
   { HCID = G4SDManager::GetSDMpointer()->GetCollectionID(hitsCollection); }
   HCE->AddHitsCollection(HCID,hitsCollection); 

  //fADC_pmt1  = 0.0;
  //fADC_pmt2  = 0.0;
  //nHits      = 0.0;
}
//-----------------------------------------------------------------------------------
G4bool TSHMSPmtSD::ProcessHits(G4Step* aStep, G4TouchableHistory*)
{
  //G4cout << "ProcessHit " << G4endl;
  G4Track *aTrack = aStep->GetTrack();
  if( !aTrack ) return false;

  G4String str = aTrack->GetDefinition()->GetParticleName();
  
  if( str == G4String("opticalphoton") ) {
  	G4StepPoint* preStepPoint = aStep->GetPreStepPoint();
  	G4StepPoint* postStepPoint = aStep->GetPostStepPoint();
  	G4TouchableHistory* theTouchable
    	= (G4TouchableHistory*)(preStepPoint->GetTouchable());
    if ( preStepPoint->GetStepStatus() == fGeomBoundary ) {
	G4ThreeVector worldPos = preStepPoint->GetPosition();
	G4ThreeVector localPos = theTouchable->GetHistory()->GetTopTransform().TransformPoint(worldPos);
	 
	 TSHMSPmtHit* aHit = new TSHMSPmtHit();
	 aHit->SetWorldPos(worldPos);
	 aHit->SetLocalPos(localPos);
	 aHit->SetTime(preStepPoint->GetGlobalTime());
	 aHit->SetTrackID(aTrack->GetTrackID());

     hitsCollection->insert(aHit);
	}
    //    G4double ephoton = aTrack->GetTotalEnergy();
    //    ephoton = ephoton * 1.0e+6; // convert to KeV
    //    G4double rand = CLHEP::RandFlat::shoot( 0.0 , 1.0 );

    //    aStep->GetTrack()->SetTrackStatus(fStopAndKill);
  }
  return true;
}

//-----------------------------------------------------------------------------------
void TSHMSPmtSD::EndOfEvent(G4HCofThisEvent*)
{
//   if (verboseLevel>0) { 
//      G4int NbHits = trackerCollection->entries();
//      G4cout << "\n-------->Hits Collection: in this event they are " << NbHits 
//             << " hits in the tracker chambers: " << G4endl;
//      for (G4int i=0;i<NbHits;i++) (*trackerCollection)[i]->Print();
//     } 

//   analysisManager->SetCerHits( nHits );
//   analysisManager->SetCerPmt1Adc( fADC_pmt1 );
//   analysisManager->SetCerPmt2Adc( fADC_pmt2 );

  //  G4cout << " ADC1,2 = " << fADC_pmt1 << " " << fADC_pmt2 << G4endl;
  //  G4cout << " End of event " << G4endl;
}
