#include "TSHMSSteppingAction.hh"
#include "TSHMSEventAction.hh"
#include "G4SteppingManager.hh"
#include "G4Step.hh"

#include <fstream>

using namespace std;

//------------------------------------------------------------------------------

TSHMSSteppingAction::TSHMSSteppingAction()
{ 
  G4cout << " #####  Stepping class created ! " << G4endl;
  analysis = TSHMSAnalysisManager::getInstance();
}
//------------------------------------------------------------------------------
void TSHMSSteppingAction::UserSteppingAction(const G4Step* aStep)
{ 
  //  G4double EdepStep = aStep->GetTotalEnergyDeposit();
  G4Track* aTrack = aStep->GetTrack();
  G4String volname = aTrack->GetVolume()->GetName();
  G4String str = aTrack->GetDefinition ()->GetParticleName();
 
  if( str == G4String("opticalphoton") ) {
    if(volname.contains("cathode")) {
      aTrack->SetKineticEnergy(0.0); 
      aStep->GetTrack()->SetTrackStatus(fStopAndKill);
	} 
    if(volname.contains("backing")) {
      aTrack->SetKineticEnergy(0.0); 
      aStep->GetTrack()->SetTrackStatus(fStopAndKill);
	}
    if( volname.contains("CerPl") ) {

      G4ThreeVector dir3D = aTrack -> GetMomentumDirection();
      string line = volname;
      int nmirr   = atoi( (line.substr( 5 , 2 )).c_str() );
      int npl     = atoi( (line.substr( 7 )).c_str() );
      G4int pid = 0; // opticalphoton
      G4StepPoint* point = aStep->GetPreStepPoint();
      if ( point->GetStepStatus() == fGeomBoundary ) {
		if( dir3D[2] < 0.0 ) {
			printTrack(aTrack, nmirr, npl, pid);
			if(!analysis->getPMTon()) {
				if((npl==50)&&(pmtHit(aTrack))) {
					npl = 0;
					printHit(aStep);
				}
			}
		}
	  }
    }
	// Here we will see how many photons miss the back plane a where
    if( volname.contains("BackPl") ) {
      G4ThreeVector pos3D = aTrack -> GetPosition();
      G4ThreeVector dir3D = aTrack -> GetMomentumDirection();
      string line = volname;
      int nmirr   = 0;	 // There will only be one back plane for the entire tube
      int npl     = 98;  // Back plane default plane number is 98

      G4int pid = 0; // opticalphoton
      G4StepPoint* point = aStep->GetPreStepPoint();
      if ( point->GetStepStatus() == fGeomBoundary ) {
		if( dir3D[2] > 0.0 ) {
			printTrack(aTrack, nmirr, npl, pid);
		}
	  }
   	}
    if( volname.contains("flange") ) {
      G4ThreeVector pos3D = aTrack -> GetPosition();
      G4ThreeVector dir3D = aTrack -> GetMomentumDirection();
      string line = volname;
      int nmirr   = 0;	 // There will only be one back plane for the entire tube
      int npl     = 97;  // Back plane default plane number is 98

      G4int pid = 0; // opticalphoton
      G4StepPoint* point = aStep->GetPreStepPoint();
      if ( point->GetStepStatus() == fGeomBoundary ) {
		if( dir3D[2] > 0.0 ) {
			printTrack(aTrack, nmirr, npl, pid);
		}
	  }
   	}
    // Here we are going to track where on the mirrors the photons are hitting
    if( volname.contains("mirr") ) {
      G4ThreeVector pos3D = aTrack -> GetPosition();
      G4ThreeVector dir3D = aTrack -> GetMomentumDirection();
      string line = volname;
      int nmirr   = atoi( (line.substr(4)).c_str() );
      int npl     = 99;	// Mirror default plane number is 99

      G4int pid = 0; // opticalphoton
      G4StepPoint* point = aStep->GetPreStepPoint();
      if ( point->GetStepStatus() == fGeomBoundary ) {
		if( dir3D[2] < 0.0 ) {
			printTrack(aTrack, nmirr, npl, pid);
		} else 
			if( dir3D[2] > 0.0 ) {
      			aTrack->SetKineticEnergy(0.0); 	// to see how many photons are being missed by mirrors
      			aStep->GetTrack()->SetTrackStatus(fStopAndKill);
	  }
   	}
   }
  } else {
    if( volname.contains("mirr") ) {  	// We want to kill non-photon particles at the mirrors
      	aTrack->SetKineticEnergy(0.0); 	// to see how many photons are being missed by mirrors
      	aStep->GetTrack()->SetTrackStatus(fStopAndKill);
  	}
 }
  	// Stop photons at the edge of the tank
    //if( aTrack->GetNextVolume()->GetName() == G4String("World") ) { 
    //  aTrack->SetKineticEnergy(0.0);
    //  aStep->GetTrack()->SetTrackStatus(fStopAndKill);
    //}
	
	/* 
	We have no need to stop photon at any plane or to track electrons at this moment
	 
    // -- Kill the photon
    if( npl ==1 && dir3D[2] < 0.0 ) { // First plane coincides with PMT front face, only photons reflected from the mirror
	aTrack->SetKineticEnergy(0.0);
	aTrack->SetTrackStatus(fStopAndKill);
      }
    
   
    if( strcmp( volname , "pmt1" )  == 0 ) aTrack->SetTrackStatus(fStopAndKill);
    if( strcmp( volname , "pmt2" )  == 0 ) aTrack->SetTrackStatus(fStopAndKill);
    if( strcmp( volname , "pmt3" )  == 0 ) aTrack->SetTrackStatus(fStopAndKill);
    if( strcmp( volname , "pmt4" )  == 0 ) aTrack->SetTrackStatus(fStopAndKill);
	
    //    G4cout << aTrack->GetVolume()->GetName() << "  " << aTrack->GetCreatorProcess()->GetProcessName() << G4endl;
  
   
  if( str == G4String("e-") ) {
    if( strcmp( volname , "cer" )  == 0 ) {
      G4ThreeVector pos3D = aTrack -> GetPosition();
      G4ThreeVector dir3D = aTrack -> GetMomentumDirection();
      G4int pid = 1; // pion 
      G4StepPoint* point = aStep->GetPreStepPoint();

	  int nmirr = -1;
	  int npl = -1;

      if ( point->GetStepStatus() == fGeomBoundary ) {

	if( dir3D[2] > 0.0 ) {
	  analysis->fout_cer << pos3D[0]/10.0 << "  " 
			     << pos3D[1]/10.0 << "  " 
			     << pos3D[2]/10.0 << "  " 
			     << nmirr         << "  "
			     << npl           << "  " 
			     << pid           << G4endl;
		
      }
    }
  }*/ 
}
// ----------------------------------------------------------------------------------------------
/**
 * printTrack - function passed G4Track pointer, int mirror number, int plane number, and int
 * particle id. Will print the position, mirror number, plane number, tracking id, and energy
 * of particle to binary .bin or text .dat file
 */
void TSHMSSteppingAction::printHit(const G4Step* aStep)
{
  G4double transport_pos3D[3];
  
  G4Track* aTrack = aStep->GetTrack();
  G4ThreeVector pos3D = aTrack->GetPosition();
  G4int TrackID = (analysis->EventNumber)*10000 + aTrack->GetTrackID();
  
  //G4PreStepPoint* preStepPoint = aStep->GetPreStepPoint();
  //G4double time = preStepPoint->GetGlobalTime();
  G4double time = -1.0*ns;
 
  transport_pos3D[0] = pos3D[0];
  transport_pos3D[1] = pos3D[1];
  transport_pos3D[2] = pos3D[2];

  analysis->fout_hit << time/ns << "  " 
		     << transport_pos3D[0]/10.0 << "  " 
		     << transport_pos3D[1]/10.0 << "  " 
		     << transport_pos3D[2]/10.0 << "  " 
		     << TrackID 	  << G4endl; 
}
void TSHMSSteppingAction::printTrack(G4Track* aTrack, G4int nmirr, G4int npl, G4int pid)
{
  G4double transport_pos3D[3];
  G4ThreeVector pos3D = aTrack -> GetPosition();
  G4double PhotonEnergy = aTrack->GetTotalEnergy();
  G4int TrackID = (analysis->EventNumber)*10000 + aTrack->GetTrackID();
 
  transport_pos3D[0] = pos3D[0];
  transport_pos3D[1] = pos3D[1];
  transport_pos3D[2] = pos3D[2];

  if(analysis->Bin) {
     //G4float pos1 = (G4float)transport_pos3D[0]/10.0; // Type cast position as a float to save room
     //G4float pos2 = (G4float)transport_pos3D[1]/10.0; // in the binary file format
     //G4float pos3 = (G4float)transport_pos3D[2]/10.0;
     G4float pos1 = (G4float)pos3D[0]/10.0; // Type cast position as a float to save room
     G4float pos2 = (G4float)pos3D[1]/10.0; // in the binary file format
     G4float pos3 = (G4float)pos3D[2]/10.0;
	 G4float photonEnergy = (G4float)PhotonEnergy;
     char cmirr = (char)nmirr; // Type cast mirror number to char 
     char cpl = (char)npl;	   // Type cast plane number to char since npl
     char cpid = (char)pid;    // Type cast particle id to char

     analysis->fout_cer.write((char*)&pos1,sizeof(float)); 
     analysis->fout_cer.write((char*)&pos2,sizeof(float)); 
     analysis->fout_cer.write((char*)&pos3,sizeof(float)); 
     analysis->fout_cer.write((char*)&photonEnergy,sizeof(float)); 
     analysis->fout_cer.write((char*)&cmirr,sizeof(char)); 
     analysis->fout_cer.write((char*)&cpl,sizeof(char)); 
     analysis->fout_cer.write((char*)&cpid,sizeof(char)); 
     analysis->fout_cer.write((char*)&TrackID,sizeof(int)); 
  } else { 	// If not a binary file then print to .dat
     analysis->fout_cer 
		     //<< transport_pos3D[0]/10.0 << "  " 
		     //<< transport_pos3D[1]/10.0 << "  " 
		     //<< transport_pos3D[2]/10.0 << "  " 
		     << transport_pos3D[0]/10.0 << "  " 
		     << transport_pos3D[1]/10.0 << "  " 
		     << transport_pos3D[2]/10.0 << "  " 
			 << PhotonEnergy  << "  "
		     << nmirr         << "  "
		     << npl           << "  " 
		     << pid           << "  " 
		     << TrackID 	  << G4endl; 
 }
}
// ----------------------------------------------------------------------------------------------
/**
 * 	pmtHit - function is passed a G4Track pointer and will determine if the particle has created
 * 	a pmt hit based on the PMT positioning. It is important to only call this function in the
 * 	case were you are in the plane that is parallel with the front of the PMT as this function
 * 	only verifies (x,y) positioning of the particle.
 */
G4bool TSHMSSteppingAction::pmtHit(G4Track* aTrack) 
{
  G4double transport_pos3D[3];
      G4ThreeVector pos3D = aTrack->GetPosition();
	  G4double PMTR = analysis->PMTR;

	  G4double PMTX, PMTY, PMTAng;
	  G4double A, B;
	  G4double X2, Y2, A2, B2;
	  G4double Area;

  	  transport_pos3D[0] = pos3D[0];
      transport_pos3D[1] = pos3D[1];
      transport_pos3D[2] = pos3D[2];

	  for(G4int ii=0; ii<4; ii++) {
		PMTX = analysis->PMT[ii][0];
		PMTY = analysis->PMT[ii][1];
		if(analysis->PMTAng[ii] > 0) {
			PMTAng = analysis->PMTAng[ii];
		} else {
			PMTAng = analysis->PMTAng[ii];
		}

		A = PMTR*cos(PMTAng*degree);  	// Since the plane is tilted then PMT will look like
		B = PMTR; 						// and ellipse in a projection on the (x,y) axies

		X2 = pow(PMTX-transport_pos3D[0],2);
		Y2 = pow(PMTY-transport_pos3D[1],2);
		A2 = A*A;
		B2 = B*B;

    	Area = (X2/A2)+(Y2/B2);	

	  	if(Area <=  1.0) {
			return true;
		}
	  }
	  return false;
}
// ----------------------------------------------------------------------------------------------
void TSHMSSteppingAction::endOfEvent() {
  
  G4cout << "END OF TRACK" << G4endl;

}
